@@ -187,6 +187,7 @@ gtk_file_list_construct(GtkFileList *file_list,
 
   icon_list->mode = mode;
   icon_list->icon_width = icon_width;
+  icon_list->selection_mode = GTK_SELECTION_SINGLE;
 
   if(path)
      file_list->path = g_strdup(path);
@@ -244,6 +245,7 @@ gtk_file_list_destroy(GtkObject *object)
   list = file_list->pixmaps;
   while(list){
     file_list->pixmaps = g_list_remove_link(file_list->pixmaps, list);
+    gtk_widget_destroy(GTK_WIDGET(list->data));
     g_list_free_1(list);
     list = file_list->pixmaps;
   }
@@ -295,8 +297,9 @@ gtk_file_list_init (GtkFileList *file_list)
 
   file_list->ntypes = 0;
 
-  for(i = 0; i <= GTK_FILE_LIST_CORE; i++)
+  for(i = 0; i <= GTK_FILE_LIST_CORE; i++){
     gtk_file_list_add_type(file_list, (const gchar **)pixmap_data[i]);
+  }
 
   types = default_types;
   while(types->extension){
@@ -370,7 +373,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
   gchar *real_path;
   GdkPixmap *pixmap;
   GdkBitmap *mask;
-  gchar **icon;
+  GtkPixmap *icon;
   gint width, height;
   GdkGC *gc;
   GList *types, *files, *list;
@@ -402,7 +405,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
            g_free(aux_path);
            aux_path = g_strdup(root);
         } else {
-           aux_path[length - 1] = '\0';
+           aux_path[length - 2] = '\0';
         }
      } else if(strcmp(aux_path + length - 3, root2) == 0){
         if(length == 3) {
@@ -412,7 +415,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
            gint i = length - 4;
            while(i >= 0){
               if(aux_path[i] == root[0]){ 
-                   aux_path[i+1] = '\0';
+                   aux_path[i] = '\0';
                    break;
               }
               i--;
@@ -426,7 +429,7 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
            gint i = length - 5;
            while(i >= 0){
               if(aux_path[i] == root[0]){ 
-                   aux_path[i+1] = '\0';
+                   aux_path[i] = '\0';
                    break;
               }
               i--;
@@ -437,9 +440,17 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
            g_free(aux_path);
            aux_path = g_strdup(root);
         } else {
-           aux_path[length - 2] = '\0';
+           aux_path[length - 3] = '\0';
         }
-     } 
+     } else if(strcmp(aux_path + length - 1, root) == 0 && length > 1){
+       aux_path[length - 1] = '\0';
+     }  
+
+     if(strlen(aux_path) == 0)
+     {
+       g_free(aux_path);
+       aux_path = g_strdup(G_DIR_SEPARATOR_S);
+     }
   }
   else
      aux_path = g_strdup(G_DIR_SEPARATOR_S);
@@ -481,11 +492,13 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
 
   gtk_icon_list_clear(icon_list);
 
+/*
   gdk_threads_enter();
+*/
 
   files = NULL;
   while((dirent=readdir(dir))!=NULL){
-        full_name = g_strconcat(real_path,dirent->d_name,NULL);
+        full_name = g_strconcat(real_path,G_DIR_SEPARATOR_S,dirent->d_name,NULL);
         file = dirent->d_name;
         show_file = TRUE;
         if(stat_subdirs){
@@ -568,11 +581,15 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
          types = types->next;
        }
 
-       icon = (gchar **)g_list_nth_data(file_list->pixmaps, type);
+       icon = GTK_PIXMAP(g_list_nth_data(file_list->pixmaps, type));
        file_item->type = type;
 
-       item = gtk_icon_list_add_from_data(icon_list, icon, 
-                                          file_item->file_name, file_item);
+       gdk_pixmap_ref(icon->pixmap);
+       gdk_bitmap_ref(icon->mask);
+       item = gtk_icon_list_add_from_pixmap(icon_list, 
+                                            icon->pixmap, 
+                                            icon->mask, 
+                                            file_item->file_name, file_item);
 
        if(file_item->is_link){
                pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, 
@@ -599,14 +616,16 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
                gdk_gc_unref(gc);
   
                gdk_pixmap_unref(pixmap);
+               gdk_bitmap_unref(mask);
        }
 
        list = list->next;
   }
 
   closedir(dir);
+/*
   gdk_threads_leave();
-
+*/
   gtk_icon_list_thaw(icon_list);
 
   g_list_free(files);
@@ -620,13 +639,13 @@ gtk_file_list_set_filter(GtkFileList *filelist, const gchar *filter)
   gtk_file_list_open_dir(filelist, filelist->path);
 }
 
-gchar *
+const gchar *
 gtk_file_list_get_path(GtkFileList *file_list)
 {
   return(file_list->path);
 }
 
-gchar *
+const gchar *
 gtk_file_list_get_filename(GtkFileList *file_list)
 {
   GList *list;
@@ -665,9 +684,32 @@ gtk_file_list_get_filetype(GtkFileList *file_list)
 
 gint
 gtk_file_list_add_type       (GtkFileList *file_list,
-                              const gchar **pixmap_data)
+                              const gchar **data)
 {
-  file_list->pixmaps = g_list_append(file_list->pixmaps, pixmap_data);
+  GdkPixmap *pixmap;
+  GdkBitmap *mask;
+  gint type;
+ 
+  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, 
+                                                 gdk_colormap_get_system(),
+                                                 &mask, NULL,
+                                                 (gchar **)data);
+
+  type = gtk_file_list_add_type_with_pixmap(file_list, pixmap, mask); 
+  gdk_pixmap_unref(pixmap);
+  gdk_bitmap_unref(mask);
+
+  return type;
+}
+
+gint
+gtk_file_list_add_type_with_pixmap (GtkFileList *file_list,
+                              GdkPixmap *pixmap, GdkBitmap *mask)
+{
+  GtkWidget *wpixmap;
+  wpixmap = gtk_pixmap_new(pixmap, mask);
+  
+  file_list->pixmaps = g_list_append(file_list->pixmaps, wpixmap);
   file_list->ntypes++;
   return (file_list->ntypes - 1);
 }
