@@ -1,5 +1,5 @@
 /* gtkiconlist - gtkiconlist widget for gtk+
- * Copyright 1999-2001  Adrian E. Feiguin <feiguin@ifir.edu.ar>
+ * Copyright 1999-DEFAULT_COL_SPACING01  Adrian E. Feiguin <feiguin@ifir.edu.ar>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -67,10 +67,10 @@ static void gtk_icon_list_class_init 		(GtkIconListClass *class);
 static void gtk_icon_list_init 			(GtkIconList *icon_list);
 static void gtk_icon_list_finalize 		(GtkObject *object);
 static void gtk_icon_list_destroy 		(GtkObject *object);
-/*
+
 static void gtk_icon_list_size_allocate		(GtkWidget *widget,
                                                  GtkAllocation *allocation);
-*/
+
 static void gtk_icon_list_realize		(GtkWidget *widget);
 static void gtk_icon_list_map			(GtkWidget *widget); 
 static void gtk_icon_list_draw 			(GtkWidget *widget, 
@@ -190,9 +190,9 @@ gtk_icon_list_class_init (GtkIconListClass *klass)
   object_class->destroy = gtk_icon_list_destroy;
 
   widget_class->realize = gtk_icon_list_realize;
-/*
+
   widget_class->size_allocate = gtk_icon_list_size_allocate;
-*/
+
   widget_class->draw = gtk_icon_list_draw;
   widget_class->map = gtk_icon_list_map;
   widget_class->expose_event = gtk_icon_list_expose;
@@ -315,7 +315,9 @@ reorder_icons(GtkIconList *iconlist)
 
   if(iconlist->freeze_count > 0) return;
 
+/*
   gdk_threads_enter();
+*/
 
   old_width = widget->allocation.width;
   old_height = widget->allocation.height;
@@ -337,7 +339,7 @@ reorder_icons(GtkIconList *iconlist)
     switch(iconlist->mode){
       case GTK_ICON_LIST_TEXT_RIGHT:
         y += vspace;
-        if(y + vspace >= old_height){
+        if(y + vspace >= old_height-DEFAULT_COL_SPACING){
                 x += hspace;
                 y = iconlist->row_spacing;
         }
@@ -346,7 +348,7 @@ reorder_icons(GtkIconList *iconlist)
       case GTK_ICON_LIST_ICON:
       default:
         x += hspace;
-        if(x + hspace >= old_width){
+        if(x + hspace >= old_width-DEFAULT_COL_SPACING){
                 x = iconlist->col_spacing;
                 y += vspace;
         }
@@ -357,7 +359,9 @@ reorder_icons(GtkIconList *iconlist)
 
    }
 
+/*
   gdk_threads_leave();
+*/
 }
 
 
@@ -369,7 +373,7 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
   GtkRequisition req;
   GtkAllocation a;
   gchar *text;
-  gint old_width, old_height, width, height;
+  gint width, height;
   gint old_x, old_y;
   gint size;
 
@@ -393,9 +397,6 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
   if(iconlist->mode == GTK_ICON_LIST_ICON) 
           req2.width = req2.height = 0;
 
-  old_width = width = GTK_WIDGET(iconlist)->allocation.width;
-  old_height = height = GTK_WIDGET(iconlist)->allocation.height;
-
   gtk_fixed_move(GTK_FIXED(iconlist), icon->pixmap, 
                  x + req1.width/2 - icon->pixmap->requisition.width/2, 
                  y + iconlist->icon_border);
@@ -406,6 +407,8 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
   icon->entry->allocation.y += (y - old_y);
   icon->entry->allocation.width = req2.width;
 
+  width = GTK_WIDGET(iconlist)->allocation.width;
+  height = GTK_WIDGET(iconlist)->allocation.height;
 
   switch(iconlist->mode){
    case GTK_ICON_LIST_TEXT_BELOW:
@@ -417,7 +420,7 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
         	        x - req2.width/2 + req1.width/2, 
                         y + req1.height + iconlist->icon_border);
 
-        if(y + req.height > height) 
+        if(y + req.height > height-DEFAULT_COL_SPACING) 
            height += req.height;
         break;
    case GTK_ICON_LIST_TEXT_RIGHT:
@@ -425,7 +428,7 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
         	        x + req1.width + iconlist->icon_border, 
                         y + req1.height/2 - req2.height/2); 
 
-        if(x + req.width > width) 
+        if(x + req.width > width-DEFAULT_COL_SPACING) 
             width += req.width;
 	break;
    case GTK_ICON_LIST_ICON:
@@ -442,13 +445,16 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
 
 }
 
-/*
+
 static void
 gtk_icon_list_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
+  GtkAllocation old = widget->allocation;
   GTK_WIDGET_CLASS(parent_class)->size_allocate(widget, allocation);
+  if(old.width != allocation->width || old.height != allocation->height)
+    reorder_icons(GTK_ICON_LIST(widget));
 }
-*/
+
 
 static void
 gtk_icon_list_realize(GtkWidget *widget)
@@ -685,6 +691,8 @@ select_icon(GtkIconList *iconlist, GtkIconListItem *item, GdkEvent *event)
 {
   gboolean veto = TRUE;
 
+  if(item == NULL) return;
+
   gtk_signal_emit(GTK_OBJECT(iconlist), signals[SELECT_ICON], item, 
                   event, &veto);                        
 
@@ -702,7 +710,7 @@ select_icon(GtkIconList *iconlist, GtkIconListItem *item, GdkEvent *event)
       gtk_entry_set_position(GTK_ENTRY(item->entry), 0);
       gtk_widget_draw(item->entry, NULL);
     }
-  }
+  } 
 
   if(item->state == GTK_STATE_SELECTED) return;
   iconlist->selection = g_list_append(iconlist->selection, item);
@@ -769,7 +777,7 @@ unselect_all(GtkIconList *iconlist)
 }
 
 GtkWidget*
-gtk_icon_list_new (guint icon_width, guint mode)
+gtk_icon_list_new (guint icon_width, GtkIconListMode mode)
 {
   GtkIconList *icon_list;
 
@@ -781,7 +789,7 @@ gtk_icon_list_new (guint icon_width, guint mode)
 }
 
 void
-gtk_icon_list_construct (GtkIconList *icon_list, guint icon_width, guint mode)
+gtk_icon_list_construct (GtkIconList *icon_list, guint icon_width, GtkIconListMode mode)
 {
   icon_list->icon_width = icon_width;
   icon_list->mode = mode;
@@ -791,7 +799,7 @@ gtk_icon_list_construct (GtkIconList *icon_list, guint icon_width, guint mode)
 }
 
 void
-gtk_icon_list_set_mode (GtkIconList *iconlist, guint mode)
+gtk_icon_list_set_mode (GtkIconList *iconlist, GtkIconListMode mode)
 {
   GList *icons;
 
@@ -812,6 +820,9 @@ gtk_icon_list_set_mode (GtkIconList *iconlist, guint mode)
         gtk_item_entry_set_justification(GTK_ITEM_ENTRY(item->entry), 
                                          GTK_JUSTIFY_CENTER);
         break;
+      case GTK_ICON_LIST_ICON:
+      default:
+        break;
     }
     
     icons = icons->next;
@@ -820,7 +831,7 @@ gtk_icon_list_set_mode (GtkIconList *iconlist, guint mode)
   reorder_icons(iconlist); 
 }
 
-guint
+GtkIconListMode
 gtk_icon_list_get_mode (GtkIconList *iconlist)
 {
   return(iconlist->mode);
@@ -839,7 +850,7 @@ gtk_icon_list_set_text_space (GtkIconList *iconlist, guint text_space)
 
     item = (GtkIconListItem *)icons->data;
     
-    GTK_ITEM_ENTRY(item->entry)->text_max_size = text_space;
+    if(item->entry) GTK_ITEM_ENTRY(item->entry)->text_max_size = text_space;
     
     icons = icons->next;
   }
@@ -917,9 +928,9 @@ entry_changed (GtkWidget *widget, gpointer data)
   if(!veto) return veto; 
 
   if(item->entry && GTK_EDITABLE(item->entry)->editable){
-    g_free(item->label);
-    item->label = g_strdup(text); 
-    g_free(item->entry_label);
+    if(item->label) g_free(item->label);
+    if(text) item->label = g_strdup(text); 
+    if(item->entry_label) g_free(item->entry_label);
     set_labels(iconlist, item, text); 
   }
 
@@ -1064,6 +1075,8 @@ gtk_icon_list_add_from_pixmap (GtkIconList *iconlist,
 {
   GtkIconListItem *item;
 
+  gdk_pixmap_ref(pixmap);
+  if(mask) gdk_bitmap_ref(mask);
   item = gtk_icon_list_real_add(iconlist, pixmap, mask, label, link);
   return item;
 }
@@ -1099,7 +1112,7 @@ gtk_icon_list_real_add (GtkIconList *iconlist,
     switch(iconlist->mode){
       case GTK_ICON_LIST_TEXT_RIGHT:
         y += vspace;
-        if(y >= height){
+        if(y >= height-DEFAULT_COL_SPACING){
                 x += hspace;
                 y = iconlist->row_spacing;
         }
@@ -1108,7 +1121,7 @@ gtk_icon_list_real_add (GtkIconList *iconlist,
       case GTK_ICON_LIST_ICON:
       default:
         x += hspace;
-        if(x >= width){
+        if(x >= width-DEFAULT_COL_SPACING){
                 x = iconlist->col_spacing;
                 y += vspace ;
         }
@@ -1136,14 +1149,11 @@ gtk_icon_list_put (GtkIconList *iconlist,
   GtkWidget *widget;
   GtkRequisition req, req1, req2;
   gint text_width;
-  gint width, height, old_width, old_height;
+  gint width, height;
   GtkAllocation alloc;
 
   widget = GTK_WIDGET(iconlist);
 
-  old_width = width = GTK_WIDGET(iconlist)->allocation.width;
-  old_height = height = GTK_WIDGET(iconlist)->allocation.height;
-
   active_icon = iconlist->active_icon;
   gtk_icon_list_set_active_icon(iconlist, NULL);
 
@@ -1176,7 +1186,8 @@ gtk_icon_list_put (GtkIconList *iconlist,
   else
           set_labels(iconlist, icon, label);
 
-  text_width = gdk_string_width(icon->entry->style->font, label);
+  text_width = 0;
+  if(label) text_width = gdk_string_width(icon->entry->style->font, label);
 
   gtk_fixed_put(GTK_FIXED(iconlist), icon->pixmap, 
                  x + req1.width/2 - icon->pixmap->requisition.width/2, 
@@ -1188,8 +1199,11 @@ gtk_icon_list_put (GtkIconList *iconlist,
   alloc.height =  req1.height;
   gtk_widget_size_allocate(icon->pixmap, &alloc);
 
+  width = GTK_WIDGET(iconlist)->allocation.width;
+  height = GTK_WIDGET(iconlist)->allocation.height;
+
   switch(iconlist->mode){
-   case GTK_ICON_LIST_TEXT_BELOW:
+      case GTK_ICON_LIST_TEXT_BELOW:
         gtk_item_entry_set_text(GTK_ITEM_ENTRY(icon->entry), icon->entry_label, 
                                 GTK_JUSTIFY_CENTER);
   	gtk_fixed_put(GTK_FIXED(iconlist), icon->entry, 
@@ -1201,10 +1215,10 @@ gtk_icon_list_put (GtkIconList *iconlist,
         alloc.height = req2.height;
         gtk_widget_size_allocate(icon->entry, &alloc);
 
-        if(y + req1.height + iconlist->icon_border + req2.height > height) 
+        if(y + req1.height + iconlist->icon_border + req2.height > height-DEFAULT_COL_SPACING) 
            height += req1.height + iconlist->icon_border + req2.height;
         break;
-   case GTK_ICON_LIST_TEXT_RIGHT:
+      case GTK_ICON_LIST_TEXT_RIGHT:
         gtk_item_entry_set_text(GTK_ITEM_ENTRY(icon->entry), icon->entry_label, 
                                 GTK_JUSTIFY_LEFT);
   	gtk_fixed_put(GTK_FIXED(iconlist), icon->entry, 
@@ -1216,11 +1230,12 @@ gtk_icon_list_put (GtkIconList *iconlist,
         alloc.height = req2.height;
         gtk_widget_size_allocate(icon->entry, &alloc);
 
-        if(x + req1.width + iconlist->icon_border + text_width > width) 
+        if(x + req1.width + iconlist->icon_border + text_width > width-DEFAULT_COL_SPACING) 
             width += req1.width + iconlist->icon_border + text_width;
 	break;
-   case GTK_ICON_LIST_ICON:
-   default: ;
+      case GTK_ICON_LIST_ICON:
+      default: 
+        break;
   }
 
   if(GTK_WIDGET_REALIZED(iconlist))
@@ -1234,7 +1249,11 @@ gtk_icon_list_put (GtkIconList *iconlist,
 
   gtk_widget_show(icon->pixmap);
 
-  iconlist->icons = g_list_insert_sorted(iconlist->icons, icon, iconlist->compare_func);
+  if(iconlist->compare_func)
+    iconlist->icons = g_list_insert_sorted(iconlist->icons, icon, iconlist->compare_func);
+  else
+    iconlist->icons = g_list_append(iconlist->icons, icon);
+
   iconlist->num_icons++;
 
   if(GTK_WIDGET_REALIZED(iconlist))
@@ -1261,6 +1280,8 @@ set_labels(GtkIconList *iconlist, GtkIconListItem *icon, const gchar *label)
   gchar *entry_label = NULL;
   gint n, space;
 
+  if(!label) return;
+
   entry_label = (gchar *)g_malloc(strlen(label) + 5);
   entry_label[0] = label[0];
   entry_label[1] = '\0';
@@ -1413,7 +1434,7 @@ gtk_icon_list_remove (GtkIconList *iconlist, GtkIconListItem *item)
   if(icons){
      if(icon->state == GTK_STATE_SELECTED) unselect_icon(iconlist, icon, NULL);
      pixmap_destroy(GTK_PIXMAP(icon->pixmap));
-     if(icon->entry)
+     if(icon->entry && iconlist->mode != GTK_ICON_LIST_ICON)
        gtk_container_remove(GTK_CONTAINER(iconlist), icon->entry);
      if(icon->pixmap)
        gtk_container_remove(GTK_CONTAINER(iconlist), icon->pixmap);
@@ -1458,12 +1479,14 @@ gtk_icon_list_clear(GtkIconList *iconlist)
 
   icons = iconlist->icons;
 
+/*
   gdk_threads_enter();
+*/
 
   while(icons){
     icon = (GtkIconListItem *) icons->data;
     pixmap_destroy(GTK_PIXMAP(icon->pixmap));
-    if(icon->entry)
+    if(icon->entry && iconlist->mode != GTK_ICON_LIST_ICON)
       gtk_container_remove(GTK_CONTAINER(iconlist), icon->entry);
     if(icon->pixmap)
       gtk_container_remove(GTK_CONTAINER(iconlist), icon->pixmap);
@@ -1483,7 +1506,9 @@ gtk_icon_list_clear(GtkIconList *iconlist)
     icons = iconlist->icons;
   }
 
+/*
   gdk_threads_leave();
+*/
 
   iconlist->icons = NULL;
   iconlist->num_icons = 0;
@@ -1551,7 +1576,7 @@ gtk_icon_list_set_label(GtkIconList *iconlist, GtkIconListItem *item, const gcha
       g_free(item->entry_label);
       item->entry_label = NULL;
   }
-  item->label = g_strdup(label);
+  if(label) item->label = g_strdup(label);
   gtk_entry_set_text(GTK_ENTRY(item->entry), label);
   set_labels(iconlist, item, label);
 }
