@@ -20,7 +20,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
-#include <malloc.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include "gtkplotdt.h"
@@ -29,9 +28,9 @@
 #define DELAUNAY_DEBUG 
 #endif
 
-
 typedef struct {
   gint a,b;
+  GtkPlotDTtriangle *t;
 } GtkPlotDTedge;
 
 static void 	gtk_plot_dt_class_init 		(GtkPlotDTClass *klass);
@@ -138,7 +137,7 @@ static void
 gtk_plot_dt_init (GtkPlotDT *data)
 {
   data->quadrilateral= TRUE;
-  data->subsampling= TRUE;
+  data->subsampling= FALSE;
   data->nodes= 0;
   data->tmp_nodes= 0;
   data->node_0= 0;
@@ -195,6 +194,7 @@ gtk_plot_dt_real_clear(GtkPlotDT *data)
   data->node_cnt= data->node_max= data->node_0= 0;
 
   gtk_plot_dt_clear_triangles(data);
+
 }
 
 static void
@@ -329,20 +329,21 @@ gtk_plot_dt_area_triangle(GtkPlotDTnode *a, GtkPlotDTnode *b, GtkPlotDTnode *c)
 	       c->x*b->y - c->y*a->x - b->x*a->y));
 }
 
-static gint
+static GtkPlotDTtriangle * 
 gtk_plot_dt_add_triangle(GtkPlotDT *data, gint a, gint b, gint c)
 {
   GtkPlotDTtriangle *t= 0;
   GtkPlotDTnode *na,*nb,*nc;
   gdouble xmin,xmax,ymin,ymax, orientation;
   if (!data) return 0;
-  if (a==b || a==c || b==c) return 0;
+  if (a==b || a==c || b==c) return NULL;
 
   t= g_new0(GtkPlotDTtriangle, 1);
-  if (!t) return 0;
+  if (!t) return NULL;
   t->a= a;
   t->b= b;
   t->c= c;
+  t->nn[0] = t->nn[1] = t->nn[2] = NULL;
 
   na= t->na= gtk_plot_dt_get_node(data,a);
   nb= t->nb= gtk_plot_dt_get_node(data,b);
@@ -378,7 +379,7 @@ gtk_plot_dt_add_triangle(GtkPlotDT *data, gint a, gint b, gint c)
   fprintf(stderr,"added triangle : %d/%d/%d c=(%g,%g) r=%g\n",a,b,c,
 	  t->ccenter.x,t->ccenter.y, t->radius);
 #endif
-  return 1;
+  return t;
 }
 
 static gint 
@@ -387,6 +388,9 @@ gtk_plot_dt_update_tmpnodes(GtkPlotDT *data)
   gint i;
   gdouble xmin,xmax,ymin,ymax,delta=0.5;
   if (!data || data->node_cnt<3) return 0;
+
+  /* determine bounding box */
+
   xmin= xmax= data->nodes[0].x;
   ymin= ymax= data->nodes[0].y;
   for (i=1; i<data->node_cnt; i++) {
@@ -402,10 +406,9 @@ gtk_plot_dt_update_tmpnodes(GtkPlotDT *data)
   /* some practical magic: have two triangles which cover _all_ the nodes */
 
   if (data->tmp_nodes) g_free(data->tmp_nodes);
-  data->tmp_nodes= g_new(GtkPlotDTnode, 8);
+  data->tmp_nodes= g_new0(GtkPlotDTnode, 8);
   data->node_0= -8;
   if (!data->tmp_nodes) return 0;
-  memset(data->tmp_nodes,0,sizeof(GtkPlotDTnode)*8);  
 
   data->tmp_nodes[0].x= xmin-delta*(xmax-xmin);
   data->tmp_nodes[0].y= ymin-delta*(ymax-ymin);
@@ -428,13 +431,24 @@ gtk_plot_dt_drop_tmpstuff(GtkPlotDT *data)
   GtkPlotDTtriangle *t;
   GList *list = NULL, *next = NULL;
   gint tri_cnt = 0;
+  gint i;
 
   if (!data) return 0;
 
+  list = data->triangles;  
+  while(list) {
+    t= (GtkPlotDTtriangle *)list->data;
+
+    for(i = 0; i < 3; i++)
+      if(t->nn[i] && (t->nn[i]->a < 0 || t->nn[i]->b < 0 || t->nn[i]->c < 0)) t->nn[i] = NULL;
+    list = list->next;
+  }
+
   list = data->triangles;  
   while(list) {
     next= list->next;
     t= (GtkPlotDTtriangle *)list->data;
+
     if (t->a<0 || t->b<0 || t->c<0) {
 #ifdef DELAUNAY_DEBUG
       fprintf(stderr,"dropping triangle %d/%d/%d\n",t->a,t->b,t->c);
@@ -449,16 +463,17 @@ gtk_plot_dt_drop_tmpstuff(GtkPlotDT *data)
   }
   g_free(data->tmp_nodes);
   data->tmp_nodes= NULL; data->node_0= 0;
+
   return tri_cnt;
 }
 
 
 /* checks if the node is inside the triangle's empty circle or not */
-static gint
+static gboolean 
 gtk_plot_dt_inside_triangle_circle(GtkPlotDTtriangle *t, GtkPlotDTnode *n)
 {
   gdouble r,x,y;
-  if (!t || !n) return 0;
+  if (!t || !n) return FALSE;
   if (t->radius < 0.0) {
     gtk_plot_dt_circle(&t->ccenter,&t->radius,t->na,t->nb,t->nc);
   }
@@ -469,9 +484,9 @@ gtk_plot_dt_inside_triangle_circle(GtkPlotDTtriangle *t, GtkPlotDTnode *n)
   fprintf(stderr,"gtk_plot_dt_inside_triangle_circle : %g < %g = %d\n",
   	  r,t->radius,(r < t->radius));
 #endif
-  if (r < t->radius) return 1;
+  if (r < t->radius) return TRUE;
   
-  return 0;
+  return FALSE;
 }
 
 /* checks if the node is inside the triangle or not */
@@ -503,7 +518,12 @@ gtk_plot_dt_inside_triangle(GtkPlotDT *data, GtkPlotDTtriangle *t,
   return B/A; /* A cannot be 0 - see gtk_plot_dt_add_triangle()! */
 }
 
-
+static gboolean
+edges_equal(GtkPlotDTedge *e1, GtkPlotDTedge *e2)
+{
+  if((e1->a == e2->a && e1->b == e2->b) || (e1->b == e2->a && e1->a == e2->b)) return TRUE;
+  return FALSE;
+}
 
 static void
 gtk_plot_dt_triangulate_insert_node(GtkPlotDT *data, GtkPlotDTnode *node)
@@ -511,10 +531,13 @@ gtk_plot_dt_triangulate_insert_node(GtkPlotDT *data, GtkPlotDTnode *node)
   gint j,k,l, delinquentes;
   GtkPlotDTtriangle *t = NULL;
   gdouble err, min;
-  GList *list = NULL, *doomed = NULL, *last = NULL;
+  GList *list = NULL, *doomed = NULL, *last = NULL, *new = NULL, *aux = NULL;
   gint num = 0;
   GtkPlotDTedge *edges = NULL;
 
+#ifdef DELAUNAY_DEBUG
+  printf("inserting node %d %f %f\n",node->id,node->x,node->y);
+#endif
   /* first, find the triangle that contains this node: 
    */
   num = 0;
@@ -533,13 +556,15 @@ gtk_plot_dt_triangulate_insert_node(GtkPlotDT *data, GtkPlotDTnode *node)
     list = list->next;
   }
   
-  if (doomed) {
+  if (doomed) { /* triangle found with node inside! */
+    GList *last_doomed;
     /* "doomed" is a new list where all removed triangles go to
      * for finding the outline of the new triangles to be inserted.
      * The first triangle to go there is the one found above.
      * Afterwards the "doomed" list is doomed to die :-)
      */
     data->triangles= g_list_remove_link(data->triangles, doomed);
+    last_doomed = g_list_last(doomed);
     delinquentes++;
 #ifdef DELAUNAY_DEBUG
     fprintf(stderr,"marking triangle %d/%d/%d doomed (min=%g)\n",
@@ -559,11 +584,11 @@ gtk_plot_dt_triangulate_insert_node(GtkPlotDT *data, GtkPlotDTnode *node)
 	/* move triangle to "doomed" list */
 	last= list->next;
 	data->triangles= g_list_remove_link(data->triangles, list);
-	/* we could append the already doomed to the single element 
-	 * to speed things up (no need to search for end of list)
-	 * but this would lead to some not-yet-understand problems...
-	 */
-	doomed= g_list_concat(doomed,list);
+        /* faster than g_list_concat */
+        list->prev = last_doomed;
+        list->next = NULL;
+        last_doomed->next = list;
+        last_doomed = list;
 	delinquentes++;
 #ifdef DELAUNAY_DEBUG
 	fprintf(stderr,"marking     also %d/%d/%d doomed\n",t->a,t->b,t->c);
@@ -576,21 +601,15 @@ gtk_plot_dt_triangulate_insert_node(GtkPlotDT *data, GtkPlotDTnode *node)
     edges= (GtkPlotDTedge*)g_malloc(sizeof(GtkPlotDTedge)*delinquentes*3);
     num= 0;
     
-    list= doomed;
+    list = doomed;
     while(list){
       t = (GtkPlotDTtriangle *)list->data;
-      edges[num].a= t->a; edges[num].b= t->b; num++;
-      edges[num].a= t->b; edges[num].b= t->c; num++;
-      edges[num].a= t->c; edges[num].b= t->a; num++;
-      /* now actually delete the "doomed" triangles,
-       * we don't need them any longer:
-       */
-      doomed= list;
-      list= g_list_remove_link(list, doomed);
-      g_free(doomed->data);
-      g_list_free_1(doomed);
+      edges[num].a= t->a; edges[num].b= t->b; edges[num].t = t; num++;
+      edges[num].a= t->b; edges[num].b= t->c; edges[num].t = t; num++;
+      edges[num].a= t->c; edges[num].b= t->a; edges[num].t = t; num++;
+      list = list->next;
     }
-    
+
 #ifdef DELAUNAY_DEBUG
     fprintf(stderr,"detected edges: ");
     for (j=0; j<num; j++)
@@ -600,9 +619,7 @@ gtk_plot_dt_triangulate_insert_node(GtkPlotDT *data, GtkPlotDTnode *node)
     for (j=0; j<num; j++) {
       l=0; 
       for (k=0; k<num; k++) 
-	if ((edges[j].a==edges[k].a && edges[j].b==edges[k].b) ||
-	    (edges[j].a==edges[k].b && edges[j].b==edges[k].a))
-	  l++;
+	if (edges_equal(&edges[j], &edges[k])) l++;
       if (l==1)
 	fprintf(stderr,"(%d %d) ",edges[j].a,edges[j].b);
     }
@@ -612,16 +629,124 @@ gtk_plot_dt_triangulate_insert_node(GtkPlotDT *data, GtkPlotDTnode *node)
     for (j=0; j<num; j++) {
       /* count occurences of this edge: */ 
       for (k=0, l=0; k<num; k++) 
-	if ((edges[j].a==edges[k].a && edges[j].b==edges[k].b) ||
-	    (edges[j].a==edges[k].b && edges[j].b==edges[k].a))
-	  l++;
+	if (edges_equal(&edges[j], &edges[k])) l++;
       /* only one occurence of an edge indicates outline edge! */
-      if (l==1) 
-	gtk_plot_dt_add_triangle(data,node->id,edges[j].a,edges[j].b);
-      
+      if (l==1){ 
+        GtkPlotDTtriangle *t;
+	t = gtk_plot_dt_add_triangle(data,node->id,edges[j].a,edges[j].b);
+        if(t) new = g_list_append(new, t);
+      }
+    }
+    if(edges) g_free(edges);
+
+
+    /* Find neighbours */
+    list = new;
+    while(list){
+      GtkPlotDTtriangle *t = (GtkPlotDTtriangle *)list->data;
+      GtkPlotDTedge t_edges[3];
+      gint l, m, n;
+      gboolean found = FALSE;
+      t_edges[0].a = t->a;
+      t_edges[0].b = t->b;
+      t_edges[1].a = t->b;
+      t_edges[1].b = t->c;
+      t_edges[2].a = t->c;
+      t_edges[2].b = t->a;
+      /* I look for the neighbours into the doomed triangles' neighbours */
+      aux = doomed;
+      while(aux){
+        GtkPlotDTedge nn[3];
+        GtkPlotDTtriangle *doomed_t = (GtkPlotDTtriangle *)aux->data;
+        /* for each doomed triangles I have 3 neighbours */
+        for(l = 0; l < 3; l++){
+          GtkPlotDTtriangle *doomed_nn = doomed_t->nn[l];
+          GList *aux2; /* verify that doomed_nn is not doomed */
+          for(aux2 = doomed; aux2; aux2 = aux2->next) 
+            if(doomed_nn == aux2->data) doomed_nn = NULL;
+          if(doomed_nn){
+            nn[0].a = doomed_nn->a;
+            nn[0].b = doomed_nn->b;
+            nn[1].a = doomed_nn->b;
+            nn[1].b = doomed_nn->c;
+            nn[2].a = doomed_nn->c;
+            nn[2].b = doomed_nn->a;
+            /* for each of the 3 neighbours I have 3 edges */
+            for(m = 0; m < 3; m++){
+              for(n = 0; n < 3; n++){
+                if(edges_equal(&nn[m], &t_edges[n])){
+                  doomed_nn->nn[m] = t;
+                  t->nn[n] = doomed_nn;
+                  found = TRUE;
+                  break; 
+                }
+              }
+              if(found) break;
+            }
+          }
+          if(found) break;
+        }
+        if(found) break;
+        aux = aux->next;
+      }
+      /* I look for the triangle's neighbours into the new triangles */
+      aux = new;
+      while(aux){
+        GtkPlotDTedge nn[3];
+        GtkPlotDTtriangle *new_t = (GtkPlotDTtriangle *)aux->data;
+        if(new_t != t){
+          nn[0].a = new_t->a;
+          nn[0].b = new_t->b;
+          nn[1].a = new_t->b;
+          nn[1].b = new_t->c;
+          nn[2].a = new_t->c;
+          nn[2].b = new_t->a;
+          /* for each of the triangles I have 3 edges */
+          found = FALSE;
+          for(m = 0; m < 3; m++){
+            for(n = 0; n < 3; n++){
+              if(edges_equal(&nn[m], &t_edges[n])){
+                new_t->nn[m] = t;
+                t->nn[n] = new_t;
+                found = TRUE;
+                break; 
+              }
+              if(found) break;
+            }
+            if(found) break;
+          }
+        }
+        aux = aux->next;
+      }
+
+      list = list->next;
     }
-    g_free(edges);
   }
+
+  /* now actually delete the "doomed" triangles,
+   * we don't need them any longer:
+   */
+  if(doomed){
+    for (list = doomed; list; list = list->next){
+      GtkPlotDTtriangle *t = (GtkPlotDTtriangle *)list->data;
+      gint i;
+      if(t->nn[0]){
+        for(i = 0; i < 3; i++)
+          if(t->nn[0]->nn[i] == t) t->nn[0]->nn[i] = NULL;
+      }
+      if(t->nn[1]){
+        for(i = 1; i < 3; i++)
+          if(t->nn[1]->nn[i] == t) t->nn[1]->nn[i] = NULL;
+      }
+      if(t->nn[2]){
+        for(i = 2; i < 3; i++)
+          if(t->nn[2]->nn[i] == t) t->nn[2]->nn[i] = NULL;
+      }
+      if (list->data) g_free(list->data);    
+    }
+    g_list_free(doomed);
+  } 
+  if(new) g_list_free(new);
 }
 
 /* Not used
@@ -679,7 +804,15 @@ gtk_plot_dt_triangle_subsample(GtkPlotDT *data,
   gint i,tmp;
   GtkPlotDTnode *n= NULL;
   gint a= 0,b= 0,c= 0,d= 0, found= 0;
-  gint u[3] = {t1->a,t1->b,t1->c}, v[3]= {t2->a,t2->b,t2->c};
+  gint u[3], v[3];
+
+  u[0] = t1->a;
+  u[1] = t1->b;
+  u[2] = t1->c;
+  v[0] = t2->a;
+  v[1] = t2->b;
+  v[2] = t2->c;
+
   for (i=0; i<4; i++) {
     if (u[0]==v[0] && u[1]==v[2]) {
       a= u[0]; b=v[1]; c=u[1]; d= u[2]; found++; break;
@@ -774,9 +907,28 @@ gtk_plot_dt_real_triangulate(GtkPlotDT *data)
       (*data->pbar) (1.0*node_idx*node_idx/max);
     }
   }
+
   /* remove auxiliary triangles: */
   tcnt= gtk_plot_dt_drop_tmpstuff(data);
 
+/*
+  {
+    GList *list = data->triangles;
+    while(list){
+      GtkPlotDTtriangle *t = (GtkPlotDTtriangle *)list->data;
+      printf("TRIANGLE %d %d %d\n",t->a,t->b,t->c);
+      if(t->nn[0])
+      printf("NN0 : %d %d %d\n",t->nn[0]->a,t->nn[0]->b,t->nn[0]->c);
+      if(t->nn[1])
+      printf("NN1 : %d %d %d\n",t->nn[1]->a,t->nn[1]->b,t->nn[1]->c);
+      if(t->nn[2])
+      printf("NN2 : %d %d %d\n",t->nn[2]->a,t->nn[2]->b,t->nn[2]->c);
+      
+      list = list->next;
+    }
+  }
+*/
+
   /* if we don't have any triangles left or we don't want subsampling,
    * we're done here 
    */
@@ -863,6 +1015,28 @@ gtk_plot_dt_compare_nodes_xy_wise(gconstpointer a, gconstpointer b)
   return 0;
 }
 
+static GtkPlotDTtriangle *
+find_neighbor(GtkPlotDT *dt, GtkPlotDTtriangle *t, gint n1, gint n2)
+{
+  GList *list;
+  GtkPlotDTtriangle *triangle;
+
+  list = dt->triangles;
+  while(list) {
+    triangle = (GtkPlotDTtriangle *)list->data;
+    if(triangle != t){
+      if(triangle->na->id == n1 && triangle->nb->id == n2) return triangle;
+      if(triangle->nb->id == n1 && triangle->na->id == n2) return triangle;
+      if(triangle->na->id == n1 && triangle->nc->id == n2) return triangle;
+      if(triangle->nc->id == n1 && triangle->na->id == n2) return triangle;
+      if(triangle->nb->id == n1 && triangle->nc->id == n2) return triangle;
+      if(triangle->nc->id == n1 && triangle->nb->id == n2) return triangle;
+    }
+    list = list->next;
+  }
+  return NULL;
+}
+
 static gboolean
 gtk_plot_dt_triangulate_tryquad(GtkPlotDT *data)
 {
@@ -937,6 +1111,7 @@ gtk_plot_dt_triangulate_tryquad(GtkPlotDT *data)
        xrow_prev= xrow_prev->next, xrow_next= xrow_next->next) {
     if (x<nx-1) {
       if (y<ny-1) {
+        GtkPlotDTtriangle *t1, *t2, *t3, *t4;
 	x0y0= xrow_prev->data;
 	x0y1= xrow_prev->next->data;
 	x1y0= xrow_next->data;
@@ -948,10 +1123,22 @@ gtk_plot_dt_triangulate_tryquad(GtkPlotDT *data)
 	center->b= x0y1->id;
 	center->c= x1y0->id;
 	center->d= x1y1->id;
-	gtk_plot_dt_add_triangle(data,-1-m,x0y0->id,x1y0->id);
-	gtk_plot_dt_add_triangle(data,-1-m,x1y0->id,x1y1->id);
-	gtk_plot_dt_add_triangle(data,-1-m,x1y1->id,x0y1->id);
-	gtk_plot_dt_add_triangle(data,-1-m,x0y1->id,x0y0->id);
+	t1 = gtk_plot_dt_add_triangle(data,-1-m,x0y0->id,x1y0->id);
+	t2 = gtk_plot_dt_add_triangle(data,-1-m,x1y0->id,x1y1->id);
+	t3 = gtk_plot_dt_add_triangle(data,-1-m,x1y1->id,x0y1->id);
+	t4 = gtk_plot_dt_add_triangle(data,-1-m,x0y1->id,x0y0->id);
+        t1->nn[0] = t4;
+        t1->nn[1] = t2;
+        t2->nn[0] = t1;
+        t2->nn[1] = t3;
+        t3->nn[0] = t2;
+        t3->nn[1] = t4;
+        t4->nn[0] = t3;
+        t4->nn[1] = t1;
+        find_neighbor(data, t1, x0y0->id, x1y0->id);
+        find_neighbor(data, t2, x1y0->id, x1y1->id);
+        find_neighbor(data, t3, x1y1->id, x0y1->id);
+        find_neighbor(data, t4, x0y1->id, x0y0->id);
 	m++;
 	y++;
       } else {
@@ -967,7 +1154,7 @@ gtk_plot_dt_triangulate_tryquad(GtkPlotDT *data)
 }
 
 void
-gtk_plot_dt_set_quatrilateral(GtkPlotDT *dt, gboolean set)
+gtk_plot_dt_set_quadrilateral(GtkPlotDT *dt, gboolean set)
 {
   gboolean old = dt->quadrilateral;
   dt->quadrilateral = set;
