@@ -35,6 +35,8 @@ static void gtk_plot_gdk_init                       (GtkPlotGdk *pc);
 static void gtk_plot_gdk_class_init                 (GtkPlotGdkClass *klass);
 static void gtk_plot_gdk_finalize                   (GtkObject *object);
 static gboolean gtk_plot_gdk_real_init              (GtkPlotPC *pc);
+static void gtk_plot_gdk_set_viewport               (GtkPlotPC *pc,
+						     gdouble w, gdouble h);
 static void gtk_plot_gdk_leave                      (GtkPlotPC *pc);
 static void gtk_plot_gdk_gsave                      (GtkPlotPC *pc);
 static void gtk_plot_gdk_grestore                   (GtkPlotPC *pc);
@@ -100,6 +102,7 @@ static void gtk_plot_gdk_draw_string                (GtkPlotPC *pc,
                                                      const gchar *text);
 static void gtk_plot_gdk_draw_pixmap                (GtkPlotPC *pc,
                                                      GdkPixmap *pixmap,
+                                                     GdkBitmap *mask,
                                                      gint xsrc, gint ysrc,
                                                      gint xdest, gint ydest,
                                                      gint width, gint height,
@@ -110,7 +113,12 @@ static GdkPixmap * scale_pixmap 		    (GdkWindow *window,
 						     GdkPixmap *pixmap, 
 						     gdouble scale_x, 
 						     gdouble scale_y);
-static gint roundint                                (gdouble x);
+static GdkBitmap * scale_bitmap 		    (GdkWindow *window, 
+						     GdkBitmap *bitmap, 
+						     gdouble scale_x, 
+						     gdouble scale_y);
+
+extern inline gint roundint                         (gdouble x);
 
 static GtkPlotPCClass *parent_class = NULL;
 
@@ -175,6 +183,7 @@ gtk_plot_gdk_class_init (GtkPlotGdkClass *klass)
 
   pc_class->init = gtk_plot_gdk_real_init;
   pc_class->leave = gtk_plot_gdk_leave;
+  pc_class->set_viewport = gtk_plot_gdk_set_viewport;
   pc_class->gsave = gtk_plot_gdk_gsave;
   pc_class->grestore = gtk_plot_gdk_grestore;
   pc_class->clip = gtk_plot_gdk_clip;
@@ -220,10 +229,13 @@ gtk_plot_gdk_construct(GtkPlotGdk *pc, GdkDrawable *drawable)
 static void
 gtk_plot_gdk_finalize (GtkObject *object)
 {
-  gdk_window_unref(GTK_PLOT_GDK(object)->window);
+  gdk_window_destroy(GTK_PLOT_GDK(object)->window);
+  GTK_PLOT_GDK(object)->window = NULL;
 
-  if(GTK_PLOT_GDK(object)->ref_count > 0)
+  if(GTK_PLOT_GDK(object)->ref_count > 0 && GTK_PLOT_GDK(object)->gc)
           gdk_gc_destroy(GTK_PLOT_GDK(object)->gc);
+
+  GTK_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
 void
@@ -243,6 +255,11 @@ gtk_plot_gdk_leave (GtkPlotPC *pc)
 {
 }
 
+static void 
+gtk_plot_gdk_set_viewport               (GtkPlotPC *pc, gdouble w, gdouble h)
+{
+}
+
 static void 
 gtk_plot_gdk_gsave                                  (GtkPlotPC *pc)
 {
@@ -258,7 +275,7 @@ gtk_plot_gdk_gsave                                  (GtkPlotPC *pc)
 static void 
 gtk_plot_gdk_grestore                                  (GtkPlotPC *pc)
 {
-  gdk_gc_unref(GTK_PLOT_GDK(pc)->gc);
+  if(GTK_PLOT_GDK(pc)->gc) gdk_gc_unref(GTK_PLOT_GDK(pc)->gc);
 
   GTK_PLOT_GDK(pc)->ref_count--;
   if(GTK_PLOT_GDK(pc)->ref_count == 0) GTK_PLOT_GDK(pc)->gc = NULL;
@@ -474,7 +491,6 @@ static gint
 drawstring(GtkPlotPC *pc,
 	   GdkBitmap *dest,
 	   GdkGC *gc,
-	   GdkColor *black, GdkColor *white,
 	   gint dx, gint dy,
 	   GtkPSFont *psfont,
 	   GdkFont *font,
@@ -484,6 +500,7 @@ drawstring(GtkPlotPC *pc,
   GdkBitmap *tmp;
   GdkFont *dfont;
   GdkImage *image;
+  GdkColor mask_color;
   gint w, h, a, d, x, y, d2;
   guint32 pixel;
 
@@ -499,24 +516,27 @@ drawstring(GtkPlotPC *pc,
 
     tmp = gdk_pixmap_new(GTK_PLOT_GDK(pc)->window, w, h, 1);
 
-    gdk_gc_set_foreground(gc, white);
+    mask_color.pixel = 0;
+    gdk_gc_set_foreground(gc, &mask_color);
     gdk_draw_rectangle(tmp, gc, TRUE, 0, 0, -1, -1);
-    gdk_gc_set_foreground(gc, black);
+    mask_color.pixel = 1;
+    gdk_gc_set_foreground(gc, &mask_color);
 
-    gdk_draw_text_wc(tmp, dfont, gc, 0, a, &wc, 1);
+    gdk_draw_text_wc(dest, dfont, gc, 0, a, &wc, 1);
 
     image = gdk_image_get(tmp, 0, 0, w, h);
 
     for (y = 0; y < h; y++) {
       for (x = 0; x < w; x++) {
 	pixel = gdk_image_get_pixel(image, x, y);
-	if (pixel == black->pixel)
+	if (pixel == 1)
 	  gdk_draw_point(dest, gc, dx + y, dy + d2 - x);
       }
     }
 
     gdk_image_destroy(image);
     gdk_pixmap_unref(tmp);
+    gdk_font_unref(font);
 
     return h;
   } else {
@@ -555,7 +575,7 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
   GdkImage *image;
   GdkGC *gc, *bitmap_gc;
   GdkColormap *colormap;
-  GdkColor white, black, mask_color;
+  GdkColor mask_color;
   GList *family = NULL;
   gint y0;
   gint old_width, old_height;
@@ -596,8 +616,8 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
     }
 
   gtk_psfont_get_families(&family, &numf);
-  font = gtk_psfont_get_gdkfont(font_name, font_height);
-  base_psfont = psfont = gtk_psfont_get_font(font_name);
+  base_psfont = psfont = gtk_psfont_get_by_name(font_name);
+  font = gtk_psfont_get_gdkfont(psfont, font_height);
   italic = psfont->italic;
   bold = psfont->bold;
   fontsize = font_height;
@@ -605,9 +625,9 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
   y0 = y = ascent;
 
   if (psfont->i18n_latinfamily) {
-    latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily, italic,
+    latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily, italic,
 					     bold);
-    latin_font = gtk_psfont_get_gdkfont(latin_psfont->psname, fontsize);
+    latin_font = gtk_psfont_get_gdkfont(latin_psfont, fontsize);
   } else {
     latin_psfont = NULL;
     latin_font = NULL;
@@ -621,12 +641,11 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
   text_bitmap = gdk_pixmap_new(GTK_PLOT_GDK(pc)->window,
                               old_width, old_height, 1);
   bitmap_gc = gdk_gc_new(text_bitmap);
-  gdk_color_white (colormap, &white);
-  gdk_gc_set_foreground(bitmap_gc, &white);
-  gdk_draw_rectangle(text_bitmap, bitmap_gc, TRUE,
-                     0, 0, -1, -1);
-  gdk_color_black (colormap, &black);
-  gdk_gc_set_foreground(bitmap_gc, &black);
+  mask_color.pixel = 0;
+  gdk_gc_set_foreground(bitmap_gc, &mask_color);
+  gdk_draw_rectangle(text_bitmap, bitmap_gc, TRUE, 0, 0, -1, -1);
+  mask_color.pixel = 1;
+  gdk_gc_set_foreground(bitmap_gc, &mask_color);
 
   while(aux && *aux != '\0' && *aux != '\n'){
    if(*aux == '\\'){
@@ -634,9 +653,9 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
      switch(*aux){
        case '0': case '1': case '2': case '3':
        case '4': case '5': case '6': case '7': case '9':
-           psfont = gtk_psfont_find_by_family((gchar *)g_list_nth_data(family, *aux-'0'), italic, bold);
+           psfont = gtk_psfont_get_by_family((gchar *)g_list_nth_data(family, *aux-'0'), italic, bold);
            gdk_font_unref(font);
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
 	   if (latin_font) {
 	     gdk_font_unref(latin_font);
 	     latin_font = NULL;
@@ -644,9 +663,9 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
            aux++;
            break;
        case '8': case 'g':
-           psfont = gtk_psfont_find_by_family("Symbol", italic, bold);
+           psfont = gtk_psfont_get_by_family("Symbol", italic, bold);
            gdk_font_unref(font);
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
 	   if (latin_font) {
 	     gdk_font_unref(latin_font);
 	     latin_font = NULL;
@@ -655,17 +674,17 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
            break;
        case 'B':
            bold = TRUE;
-           psfont = gtk_psfont_find_by_family(psfont->family, italic, bold);
+           psfont = gtk_psfont_get_by_family(psfont->family, italic, bold);
            gdk_font_unref(font);
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
 	   if (latin_font) {
 	     gdk_font_unref(latin_font);
 	     latin_font = NULL;
 	   }
 	   if (psfont->i18n_latinfamily) {
-	     latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily,
+	     latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily,
 						      italic, bold);
-	     latin_font = gtk_psfont_get_gdkfont(latin_psfont->psname,
+	     latin_font = gtk_psfont_get_gdkfont(latin_psfont,
 						 fontsize);
 	   }
            aux++;
@@ -699,17 +718,17 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
            break;
        case 'i':
            italic = TRUE;
-           psfont = gtk_psfont_find_by_family(psfont->family, italic, bold);
+           psfont = gtk_psfont_get_by_family(psfont->family, italic, bold);
            gdk_font_unref(font);
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
 	   if (latin_font) {
 	     gdk_font_unref(latin_font);
 	     latin_font = NULL;
 	   }
 	   if (psfont->i18n_latinfamily) {
-	     latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily,
+	     latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily,
 						      italic, bold);
-	     latin_font = gtk_psfont_get_gdkfont(latin_psfont->psname,
+	     latin_font = gtk_psfont_get_gdkfont(latin_psfont,
 						 fontsize);
 	   }
            aux++;
@@ -717,16 +736,16 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
        case 'S': case '^':
            fontsize = (int)((gdouble)fontsize * 0.6 + 0.5);
            gdk_font_unref(font);
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
            y -= font->ascent;
 	   if (latin_font) {
 	     gdk_font_unref(latin_font);
 	     latin_font = NULL;
 	   }
 	   if (psfont->i18n_latinfamily) {
-	     latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily,
+	     latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily,
 						      italic, bold);
-	     latin_font = gtk_psfont_get_gdkfont(latin_psfont->psname,
+	     latin_font = gtk_psfont_get_gdkfont(latin_psfont,
 						 fontsize);
 	   }
            aux++;
@@ -734,16 +753,16 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
        case 's': case '_':
            fontsize = (int)((gdouble)fontsize * 0.6 + 0.5);
            gdk_font_unref(font);
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
            y += font->descent;
 	   if (latin_font) {
 	     gdk_font_unref(latin_font);
 	     latin_font = NULL;
 	   }
 	   if (psfont->i18n_latinfamily) {
-	     latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily,
+	     latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily,
 						      italic, bold);
-	     latin_font = gtk_psfont_get_gdkfont(latin_psfont->psname,
+	     latin_font = gtk_psfont_get_gdkfont(latin_psfont,
 						 fontsize);
 	   }
            aux++;
@@ -751,15 +770,15 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
        case '+':
            fontsize += 3;
            gdk_font_unref(font);
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
 	   if (latin_font) {
 	     gdk_font_unref(latin_font);
 	     latin_font = NULL;
 	   }
 	   if (psfont->i18n_latinfamily) {
-	     latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily,
+	     latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily,
 						      italic, bold);
-	     latin_font = gtk_psfont_get_gdkfont(latin_psfont->psname,
+	     latin_font = gtk_psfont_get_gdkfont(latin_psfont,
 						 fontsize);
 	   }
            aux++;
@@ -767,15 +786,15 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
        case '-':
            fontsize -= 3;
            gdk_font_unref(font);
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
 	   if (latin_font) {
 	     gdk_font_unref(latin_font);
 	     latin_font = NULL;
 	   }
 	   if (psfont->i18n_latinfamily) {
-	     latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily,
+	     latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily,
 						      italic, bold);
-	     latin_font = gtk_psfont_get_gdkfont(latin_psfont->psname,
+	     latin_font = gtk_psfont_get_gdkfont(latin_psfont,
 						 fontsize);
 	   }
            aux++;
@@ -784,7 +803,7 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
 	   psfont = base_psfont;
            gdk_font_unref(font);
            fontsize = font_height;
-           font = gtk_psfont_get_gdkfont(psfont->psname, fontsize);
+           font = gtk_psfont_get_gdkfont(psfont, fontsize);
            y = y0;
            italic = psfont->italic;
            bold = psfont->bold;
@@ -793,9 +812,9 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
 	     latin_font = NULL;
 	   }
 	   if (psfont->i18n_latinfamily) {
-	     latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily,
+	     latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily,
 						      italic, bold);
-	     latin_font = gtk_psfont_get_gdkfont(latin_psfont->psname,
+	     latin_font = gtk_psfont_get_gdkfont(latin_psfont,
 						 fontsize);
 	   }
            aux++;
@@ -819,16 +838,17 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
            break;
        default:
            if(aux && *aux != '\0' && *aux !='\n'){
-	     x += drawstring(pc, text_bitmap, bitmap_gc, &black, &white, x, y,
+	     x += drawstring(pc, text_bitmap, bitmap_gc, x, y,
 			     psfont, font, latin_font, *aux);
 	     lastchar = aux;
 	     aux++;
+
 	   }
 	   break;
      }
    } else {
      if(aux && *aux != '\0' && *aux !='\n'){
-       x += drawstring(pc, text_bitmap, bitmap_gc, &black, &white, x, y,
+       x += drawstring(pc, text_bitmap, bitmap_gc, x, y,
 		       psfont, font, latin_font, *aux);
        lastchar = aux;
        aux++;
@@ -840,11 +860,9 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
 
   /* initializing clip mask bitmap - ajd */
   text_mask = gdk_pixmap_new(GTK_PLOT_GDK(pc)->window, width, height, 1);
-  mask_color = white;
   mask_color.pixel = 0;
   gdk_gc_set_foreground(bitmap_gc, &mask_color);
   gdk_draw_rectangle(text_mask, bitmap_gc, TRUE, 0, 0, -1, -1);
-  mask_color = black;
   mask_color.pixel = 1;
   gdk_gc_set_foreground(bitmap_gc, &mask_color);
 
@@ -853,7 +871,7 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
   for(y = 0; y < old_height; y++)
       for(x = 0; x < old_width; x++)
          {
-           if( black.pixel == gdk_image_get_pixel(image, x, y) ){
+           if( gdk_image_get_pixel(image, x, y) == 1 ){
            switch(angle){
             case 0:
                 xp = x;
@@ -913,10 +931,11 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
 
   gdk_pixmap_unref(text_pixmap);
   gdk_bitmap_unref(text_mask);
+  gdk_bitmap_unref(text_bitmap);
   gdk_font_unref(font);
-  if (latin_font) gdk_font_unref(latin_font);
+  if (latin_font) 
+    gdk_font_unref(latin_font);
   gdk_gc_unref(bitmap_gc);
-  gdk_pixmap_unref(text_bitmap);
 
 
 /* border */
@@ -951,6 +970,7 @@ gtk_plot_gdk_draw_string                        (GtkPlotPC *pc,
 
 static void gtk_plot_gdk_draw_pixmap                (GtkPlotPC *pc,
                                                      GdkPixmap *pixmap,
+                                                     GdkBitmap *mask,
                                                      gint xsrc, gint ysrc,
                                                      gint xdest, gint ydest,
                                                      gint width,
@@ -960,6 +980,7 @@ static void gtk_plot_gdk_draw_pixmap                (GtkPlotPC *pc,
 {
   GdkGC *gc;
   GdkPixmap *new_pixmap;
+  GdkBitmap *new_mask = NULL;
 
   if(!GTK_PLOT_GDK(pc)->drawable) return;
   if(!GTK_PLOT_GDK(pc)->window) return;
@@ -970,9 +991,16 @@ static void gtk_plot_gdk_draw_pixmap                (GtkPlotPC *pc,
   if(!gc) return;
 
   new_pixmap = scale_pixmap(GTK_PLOT_GDK(pc)->window, pixmap, scale_x, scale_y);
+  
+  if(mask)
+    new_mask = scale_bitmap(GTK_PLOT_GDK(pc)->window, mask, scale_x, scale_y);
+
+  gtk_plot_pc_clip_mask(pc, xdest, ydest, new_mask);
   gdk_draw_pixmap(GTK_PLOT_GDK(pc)->drawable, gc, new_pixmap,
                   xsrc, ysrc, xdest, ydest, width*scale_x, height*scale_y);
+  gtk_plot_pc_clip_mask(pc, xdest, ydest, NULL);
 
+  if(new_mask) gdk_bitmap_unref(new_mask);
   gdk_pixmap_unref(new_pixmap);
 }
 
@@ -1036,8 +1064,75 @@ scale_pixmap (GdkWindow *window, GdkPixmap *pixmap, gdouble scale_x, gdouble sca
   return new_pixmap;
 }
 
-static gint
-roundint(gdouble x)
+static GdkBitmap *
+scale_bitmap (GdkWindow *window, GdkBitmap *bitmap, gdouble scale_x, gdouble scale_y)
 {
- return (x+.50999999471);
+  GdkGC *gc;
+  GdkColormap *colormap;
+  GdkColorContext *cc;
+  GdkVisual *visual;
+  GdkImage *image;
+  GdkBitmap *new_bitmap;
+  gint x, y, width, height, new_width, new_height;
+  GdkColor color;
+
+  if(!bitmap) return NULL;
+  if(!window) return NULL;
+
+  gc = gdk_gc_new(bitmap);
+  colormap = gdk_colormap_get_system ();
+  visual = gdk_visual_get_system ();
+  cc = gdk_color_context_new(visual, colormap);
+  gdk_window_get_size(bitmap, &width, &height);
+
+
+  if(scale_x == 1.0 && scale_y == 1.0){
+    new_bitmap = gdk_pixmap_new(window, width, height, 1);
+    color.pixel = 0;
+    gdk_gc_set_foreground(gc, &color);
+    gdk_draw_rectangle(new_bitmap, gc, TRUE, 0, 0, width, height); 
+    color.pixel = 1;
+    gdk_gc_set_foreground(gc, &color);
+
+    gdk_draw_pixmap(new_bitmap,
+                    gc,
+                    bitmap,
+                    0, 0,
+                    0, 0,
+                    width, height);
+    return new_bitmap;
+  }
+
+  new_width = roundint(width * scale_x);
+  new_height = roundint(height * scale_y);
+  new_bitmap = gdk_pixmap_new(window, new_width, new_height, 1);
+
+  color.pixel = 0;
+  gdk_gc_set_foreground(gc, &color);
+  gdk_draw_rectangle(new_bitmap, gc, TRUE, 0, 0, width, height); 
+  color.pixel = 1;
+  gdk_gc_set_foreground(gc, &color);
+
+  image = gdk_image_get(bitmap,
+                        0, 0,
+                        width, height);
+
+  for(x = 0; x < new_width; x++){
+    for(y = 0; y < new_height; y++){
+      gint px, py;
+      gulong pixel;
+
+      px = MIN(roundint(x / scale_x), width - 1);
+      py = MIN(roundint(y / scale_y), height - 1);
+
+      pixel = gdk_image_get_pixel(image, px, py);
+
+      if(pixel != 0) gdk_draw_point(new_bitmap, gc, x, y);
+    }
+  }
+
+  gdk_image_destroy(image);
+  gdk_color_context_free(cc);
+  return new_bitmap;
 }
+
