@@ -34,7 +34,6 @@
 #include <ctype.h>
 #include <time.h>
 #include <gtk/gtk.h>
-#include <wctype.h>
 #include <locale.h>
 
 #include "gtkplot.h"
@@ -155,6 +154,7 @@ gtk_plot_ps_init (GtkPlotPS *ps)
 {
   ps->psname = NULL;
   ps->gsaved = FALSE;
+  GTK_PLOT_PC(ps)->use_pixmap = FALSE;
 }
 
 
@@ -356,8 +356,8 @@ static void pssetlineattr			(GtkPlotPC *pc,
     FILE *psout = GTK_PLOT_PS(pc)->psfile;
 
     fprintf(psout, "%g slw\n", line_width);
-    fprintf(psout, "%d slc\n", abs(cap_style - 1));
-    fprintf(psout, "%d slj\n", join_style);
+    fprintf(psout, "%i slc\n", abs((int)cap_style - 1));
+    fprintf(psout, "%i slj\n", join_style);
 
     if(line_style == 0)
             fprintf(psout,"[] 0 sd\n");  /* solid line */
@@ -738,10 +738,13 @@ psoutputstring (GtkPlotPC *pc,
   gint curcode = 0, code; /* 0..neither 1..latin 2..i18n */
   gchar begin[] = { 0, '(', '<' };
   gchar end[] = { 0, ')', '>' };
-  GtkPSFont *fonts[] = { NULL, latin_psfont, psfont };
+  GtkPSFont *fonts[3];
   FILE *out = GTK_PLOT_PS(pc)->psfile;
   gchar *mbs, *c;
-  
+ 
+  fonts[0] = NULL; 
+  fonts[1] = latin_psfont; 
+  fonts[2] = psfont; 
 
   p = wstring;
   
@@ -832,14 +835,14 @@ psdrawstring	(GtkPlotPC *pc,
   psout = GTK_PLOT_PS(pc)->psfile;
 
   gtk_psfont_get_families(&family, &numf);
-  base_psfont = psfont = gtk_psfont_get_font(font);
+  base_psfont = psfont = gtk_psfont_get_by_name(font);
   italic = psfont->italic;
   bold = psfont->bold;
 
   currfont = psfont->psname;
 
   if (psfont->i18n_latinfamily) {
-    latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily, italic,
+    latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily, italic,
 					     bold);
   }
 
@@ -975,7 +978,7 @@ psdrawstring	(GtkPlotPC *pc,
 		  psoutputstring(pc, psfont, latin_psfont, (gint)scale,
 				 curstr, "show");
                   curcnt = 0;
-                  psfont = gtk_psfont_find_by_family((gchar *)g_list_nth_data(family, *aux-'0'), italic, bold);
+                  psfont = gtk_psfont_get_by_family((gchar *)g_list_nth_data(family, *aux-'0'), italic, bold);
 		  aux++;
                   break;
            case '8':case 'g':
@@ -983,7 +986,7 @@ psdrawstring	(GtkPlotPC *pc,
                   psoutputstring(pc, psfont, latin_psfont, (gint)scale,
 				 curstr, "show");
                   curcnt = 0;
-                  psfont = gtk_psfont_find_by_family("Symbol", italic, bold);
+                  psfont = gtk_psfont_get_by_family("Symbol", italic, bold);
                   aux++;
                   break;
            case 'B':
@@ -992,9 +995,9 @@ psdrawstring	(GtkPlotPC *pc,
 				 curstr, "show");
                   curcnt = 0;
   		  bold = TRUE;
-                  psfont = gtk_psfont_find_by_family(psfont->family, italic, bold);
+                  psfont = gtk_psfont_get_by_family(psfont->family, italic, bold);
 		  if (psfont->i18n_latinfamily)
-		    latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily, italic, bold);
+		    latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily, italic, bold);
 		  aux++;
                   break;
            case 'x':
@@ -1028,9 +1031,9 @@ psdrawstring	(GtkPlotPC *pc,
 				 curstr, "show");
                   curcnt = 0;
 		  italic = TRUE;
-                  psfont = gtk_psfont_find_by_family(psfont->family, italic, bold);
+                  psfont = gtk_psfont_get_by_family(psfont->family, italic, bold);
 		  if (psfont->i18n_latinfamily)
-		    latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily, italic, bold);
+		    latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily, italic, bold);
 		  aux++;
                   break;
            case 's':case '_':
@@ -1062,7 +1065,7 @@ psdrawstring	(GtkPlotPC *pc,
 		  italic = psfont->italic;
 		  bold = psfont->bold;
 		  if (psfont->i18n_latinfamily) {
-		    latin_psfont = gtk_psfont_find_by_family(psfont->i18n_latinfamily,
+		    latin_psfont = gtk_psfont_get_by_family(psfont->i18n_latinfamily,
 							     italic, bold);
 		  }
                   scale = font_height;
@@ -1173,7 +1176,9 @@ psgrestore(GtkPlotPC *pc)
 
   psout = ps->psfile;
 
+/*
   if(!ps->gsaved) return;
+*/
 
   fprintf(psout,"grestore\n");
   ps->gsaved = FALSE;
