@@ -7,6 +7,12 @@
 #include "gtkplotdata.h"
 #include "gtkplotbar.h"
 #include "gtkplotcanvas.h"
+#include "gtkplotcanvastext.h"
+#include "gtkplotcanvasline.h"
+#include "gtkplotcanvasellipse.h"
+#include "gtkplotcanvasrectangle.h"
+#include "gtkplotcanvasplot.h"
+#include "gtkplotcanvaspixmap.h"
 #include "gtkplotps.h"
 #include "gtkplotprint.h"
 
@@ -15,6 +21,7 @@ GtkWidget **plots;
 GtkWidget **buttons;
 GtkPlotData *dataset[5];
 gint nlayers = 0;
+GtkWidget *active_plot;
 
 static void put_child(GtkPlotCanvas *canvas, gdouble x, gdouble y);
 
@@ -45,106 +52,102 @@ gdouble gaussian(GtkPlot *plot, GtkPlotData *data, gdouble x, gboolean *err)
  return y;
 }
 
+/*
 gint
-move_item(GtkWidget *widget, GtkPlotCanvasChild *item, 
+move_item(GtkWidget *widget, GtkPlotCanvasChild *item,
           gdouble x, gdouble y, gpointer data)
 {
   GtkPlotCanvas *canvas = NULL;
-
+                                                                                
   canvas = GTK_PLOT_CANVAS(widget);
-
-  if(item->type == GTK_PLOT_CANVAS_DATA){
-        printf("MOVING DATA\n");
-        printf("Active point: %d -> %f %f\n", 
-                canvas->active_point, x, y);
-	canvas->active_data->x[canvas->active_point] = x;
-	canvas->active_data->y[canvas->active_point] = y;
+                                                                                
+  if(GTK_IS_PLOT_CANVAS_PLOT(item) && GTK_PLOT_CANVAS_PLOT(item)->pos == GTK_PLOT_CANVAS_PLOT_IN_DATA){
+      gdouble *array_x = NULL;
+      gdouble *array_y = NULL;
+      gint n;
+      array_x = gtk_plot_data_get_x(GTK_PLOT_CANVAS_PLOT(item)->data, &n);
+      array_y = gtk_plot_data_get_y(GTK_PLOT_CANVAS_PLOT(item)->data, &n);
+      printf("MOVING DATA\n");
+      printf("Active point: %d", GTK_PLOT_CANVAS_PLOT(item)->datapoint);
   }
- 
+                                                                                
   return TRUE;
 }
+*/
 
 gint
-select_item(GtkWidget *widget, GdkEvent *event, GtkPlotCanvasChild *item, 
+select_item(GtkWidget *widget, GdkEvent *event, GtkPlotCanvasChild *child, 
             gpointer data)
 {
   GtkWidget **widget_list = NULL;
   GtkWidget *active_widget = NULL;
-  GtkWidget *canvas = NULL;
   gint n = 0;
+  gdouble *x = NULL, *y = NULL;
 
-  switch(item->type){
-    case GTK_PLOT_CANVAS_TEXT:
+  if(GTK_IS_PLOT_CANVAS_TEXT(child))
         printf("Item selected: TEXT\n");
-        break;
-    case GTK_PLOT_CANVAS_TITLE:
+  if(GTK_IS_PLOT_CANVAS_PIXMAP(child))
+        printf("Item selected: PIXMAP\n");
+  if(GTK_IS_PLOT_CANVAS_RECTANGLE(child))
+        printf("Item selected: RECTANGLE\n");
+  if(GTK_IS_PLOT_CANVAS_ELLIPSE(child))
+        printf("Item selected: ELLIPSE\n");
+  if(GTK_IS_PLOT_CANVAS_LINE(child))
+        printf("Item selected: LINE\n");
+  if(GTK_IS_PLOT_CANVAS_PLOT(child)){
+    switch(GTK_PLOT_CANVAS_PLOT(child)->pos){
+      case GTK_PLOT_CANVAS_PLOT_IN_TITLE:
         printf("Item selected: TITLE\n");
         break;
-    case GTK_PLOT_CANVAS_LEGENDS:
+      case GTK_PLOT_CANVAS_PLOT_IN_LEGENDS:
         printf("Item selected: LEGENDS\n");
         break;
-    case GTK_PLOT_CANVAS_PLOT:
+      case GTK_PLOT_CANVAS_PLOT_IN_PLOT:
         printf("Item selected: PLOT\n");
         break;
-    case GTK_PLOT_CANVAS_AXIS:
+      case GTK_PLOT_CANVAS_PLOT_IN_AXIS:
         printf("Item selected: AXIS\n");
         break;
-    case GTK_PLOT_CANVAS_PIXMAP:
-        printf("Item selected: PIXMAP\n");
-        break;
-    case GTK_PLOT_CANVAS_MARKER:
+      case GTK_PLOT_CANVAS_PLOT_IN_MARKER:
         printf("Item selected: MARKER\n");
         break;
-    case GTK_PLOT_CANVAS_DATA:
+      case GTK_PLOT_CANVAS_PLOT_IN_GRADIENT:
+        printf("Item selected: GRADIENT\n");
+        break;
+      case GTK_PLOT_CANVAS_PLOT_IN_DATA:
+        x = gtk_plot_data_get_x(GTK_PLOT_CANVAS_PLOT(child)->data, &n); 
+        y = gtk_plot_data_get_y(GTK_PLOT_CANVAS_PLOT(child)->data, &n); 
+        n = GTK_PLOT_CANVAS_PLOT(child)->datapoint;
         printf("Item selected: DATA\n");
         printf("Active point: %d -> %f %f\n", 
-                GTK_PLOT_CANVAS(widget)->active_point,
-                GTK_PLOT_CANVAS(widget)->active_x, 
-                GTK_PLOT_CANVAS(widget)->active_y); 
-
-/*
-        gtk_plot_data_add_marker(GTK_PLOT_CANVAS(widget)->active_data,
-                                 GTK_PLOT_CANVAS(widget)->active_point);
-        GTK_PLOT_CANVAS_UNSET_FLAGS(GTK_PLOT_CANVAS(widget),
-				    GTK_PLOT_CANVAS_CAN_SELECT_POINT|
-				    GTK_PLOT_CANVAS_CAN_DND_POINT);
-        gtk_plot_canvas_paint(GTK_PLOT_CANVAS(widget));
-        gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(widget));
-*/
-
+                GTK_PLOT_CANVAS_PLOT(child)->datapoint, x[n], y[n]);
         break;
-    case GTK_PLOT_CANVAS_NONE:
-        printf("Item selected: NONE\n");
+      default:
         break;
-    case GTK_PLOT_CANVAS_CUSTOM:
-        printf("This is a custom child. Isn't it neat?\n");
-        break;
-    default:
-        ;
-  }
+    }
 
-  canvas = GTK_WIDGET(widget);
-  widget_list = plots;
-  active_widget = GTK_WIDGET(gtk_plot_canvas_get_active_plot(GTK_PLOT_CANVAS(widget)));
-
-  while(n < nlayers)
-    {
-      if(widget_list[n] == active_widget){
-            gtk_plot_canvas_set_active_plot(GTK_PLOT_CANVAS(canvas),
-                                            GTK_PLOT(plots[n]));  
+    widget_list = plots;
+    active_widget = GTK_WIDGET(GTK_PLOT_CANVAS_PLOT(child)->plot);
+    n = 0;
+                                                                                
+    while(n < nlayers)
+      {
+        if(widget_list[n] == active_widget){
+            active_plot = plots[n];
             GTK_BUTTON(buttons[n])->button_down = TRUE;
             GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
             gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
-      }else{
+        }else{
             GTK_BUTTON(buttons[n])->button_down = FALSE;
             GTK_TOGGLE_BUTTON(buttons[n])->active = FALSE;
             gtk_widget_set_state(buttons[n], GTK_STATE_NORMAL);
+        }
+        gtk_widget_queue_draw(buttons[n]);
+                                                                                
+        n++;
       }
-      gtk_widget_queue_draw(buttons[n]);
-      
-      n++;
-    }
 
+  }
 
   return TRUE;
 }
@@ -164,8 +167,7 @@ activate_plot(GtkWidget *widget, gpointer data)
   while(n < nlayers)
     {
       if(widget_list[n] == active_widget){
-            gtk_plot_canvas_set_active_plot(GTK_PLOT_CANVAS(canvas),
-                                            GTK_PLOT(plots[n]));  
+            active_plot = plots[n];  
             GTK_BUTTON(buttons[n])->button_down = TRUE;
             GTK_TOGGLE_BUTTON(buttons[n])->active = TRUE;
             gtk_widget_set_state(buttons[n], GTK_STATE_ACTIVE);
@@ -226,7 +228,7 @@ my_tick_label(GtkPlotAxis *axis, gdouble *tick_value, gchar *label, gpointer dat
 }
 
 void
-build_example1(GtkWidget *active_plot)
+build_example1(GtkWidget *plot)
 {
  GdkColor color;
  GtkPlotAxis *axis;
@@ -243,25 +245,31 @@ build_example1(GtkWidget *active_plot)
 
  /* CUSTOM TICK LABELS */
 
- gtk_plot_axis_use_custom_tick_labels(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, TRUE);
- axis = gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT);
+ gtk_plot_axis_use_custom_tick_labels(gtk_plot_get_axis(GTK_PLOT(plot), GTK_PLOT_AXIS_RIGHT), TRUE);
+ axis = gtk_plot_get_axis(GTK_PLOT(plot), GTK_PLOT_AXIS_RIGHT);
  gtk_signal_connect(GTK_OBJECT(axis), "tick_label", 
                     GTK_SIGNAL_FUNC(my_tick_label), NULL);
 
  dataset[0] = GTK_PLOT_DATA(gtk_plot_data_new());
- gtk_plot_add_data(GTK_PLOT(active_plot), dataset[0]);
+ gtk_plot_add_data(GTK_PLOT(plot), dataset[0]);
  gtk_widget_show(GTK_WIDGET(dataset[0]));
  gtk_plot_data_set_points(dataset[0], px1, py1, dx1, dy1, 6);
+
+ gtk_plot_data_add_marker(dataset[0], 3);
+/*
+ gtk_plot_data_gradient_set_visible(dataset[0], TRUE);
+*/
+
  gdk_color_parse("red", &color);
  gdk_color_alloc(gdk_colormap_get_system(), &color); 
 
  gtk_plot_data_set_symbol(dataset[0],
-                             GTK_PLOT_SYMBOL_DIAMOND,
-			     GTK_PLOT_SYMBOL_EMPTY,
-                             10, 2, &color, &color);
+                          GTK_PLOT_SYMBOL_DIAMOND,
+			  GTK_PLOT_SYMBOL_EMPTY,
+                          10, 2, &color, &color);
  gtk_plot_data_set_line_attributes(dataset[0],
-                                      GTK_PLOT_LINE_SOLID,
-                                      1, &color);
+                                   GTK_PLOT_LINE_SOLID,
+                                   0, 0, 1, &color);
 
  gtk_plot_data_set_connector(dataset[0], GTK_PLOT_CONNECT_SPLINE);
 
@@ -269,26 +277,26 @@ build_example1(GtkWidget *active_plot)
  gtk_plot_data_set_legend(dataset[0], "Spline + EY");
 
  dataset[3] = GTK_PLOT_DATA(gtk_plot_data_new());
- gtk_plot_add_data(GTK_PLOT(active_plot), dataset[3]);
+ gtk_plot_add_data(GTK_PLOT(plot), dataset[3]);
  gtk_widget_show(GTK_WIDGET(dataset[3]));
  gtk_plot_data_set_points(dataset[3], px2, py2, dx2, dy2, 6);
  gtk_plot_data_set_symbol(dataset[3],
                              GTK_PLOT_SYMBOL_SQUARE,
 			     GTK_PLOT_SYMBOL_OPAQUE,
                              8, 2, 
-                             &active_plot->style->black,
-                             &active_plot->style->black);
+                             &plot->style->black,
+                             &plot->style->black);
  gtk_plot_data_set_line_attributes(dataset[3],
-                                      GTK_PLOT_LINE_SOLID,
-                                      4, &color);
+                                   GTK_PLOT_LINE_SOLID,
+                                   0, 0, 4, &color);
  gtk_plot_data_set_connector(dataset[3], GTK_PLOT_CONNECT_STRAIGHT);
 
  gtk_plot_data_set_x_attributes(dataset[3], 
                                 GTK_PLOT_LINE_SOLID,
-                                0, &active_plot->style->black);
+                                0, 0, 0, &plot->style->black);
  gtk_plot_data_set_y_attributes(dataset[3], 
                                 GTK_PLOT_LINE_SOLID,
-                                0, &active_plot->style->black);
+                                0, 0, 0, &plot->style->black);
 
  gtk_plot_data_set_legend(dataset[3], "Line + Symbol");
 
@@ -296,30 +304,30 @@ build_example1(GtkWidget *active_plot)
  gdk_color_parse("blue", &color);
  gdk_color_alloc(gdk_colormap_get_system(), &color); 
 
- dataset[1] = gtk_plot_add_function(GTK_PLOT(active_plot), (GtkPlotFunc)function);
+ dataset[1] = gtk_plot_add_function(GTK_PLOT(plot), (GtkPlotFunc)function);
  gtk_widget_show(GTK_WIDGET(dataset[1]));
  gtk_plot_data_set_line_attributes(dataset[1],
-                                      GTK_PLOT_LINE_SOLID,
-                                      0, &color);
+                                   GTK_PLOT_LINE_SOLID,
+                                   0, 0, 0, &color);
 
  gtk_plot_data_set_legend(dataset[1], "Function Plot");
 }
 
 void
-build_example2(GtkWidget *active_plot)
+build_example2(GtkWidget *plot)
 {
  GdkColor color;
  static double px2[] = {.1, .2, .3, .4, .5, .6, .7, .8};
  static double py2[] = {.012, .067, .24, .5, .65, .5, .24, .067};
  static double dx2[] = {.1, .1, .1, .1, .1, .1, .1, .1};
 
- dataset[4] = gtk_plot_add_function(GTK_PLOT(active_plot), (GtkPlotFunc)gaussian);
+ dataset[4] = gtk_plot_add_function(GTK_PLOT(plot), (GtkPlotFunc)gaussian);
  gtk_widget_show(GTK_WIDGET(dataset[4]));
  gdk_color_parse("dark green", &color);
  gdk_color_alloc(gdk_colormap_get_system(), &color); 
  gtk_plot_data_set_line_attributes(dataset[4],
-                                      GTK_PLOT_LINE_DASHED,
-                                      2, &color);
+                                   GTK_PLOT_LINE_DASHED,
+                                   0, 0, 2, &color);
 
  gtk_plot_data_set_legend(dataset[4], "Gaussian");
 
@@ -327,11 +335,11 @@ build_example2(GtkWidget *active_plot)
  gdk_color_parse("blue", &color);
  gdk_color_alloc(gdk_colormap_get_system(), &color); 
 /*
- GTK_PLOT(active_plot)->xscale = GTK_PLOT_SCALE_LOG10;
+ GTK_PLOT(plot)->xscale = GTK_PLOT_SCALE_LOG10;
 */
 
  dataset[2] = GTK_PLOT_DATA(gtk_plot_bar_new(GTK_ORIENTATION_VERTICAL));
- gtk_plot_add_data(GTK_PLOT(active_plot), dataset[2]);
+ gtk_plot_add_data(GTK_PLOT(plot), dataset[2]);
  gtk_widget_show(GTK_WIDGET(dataset[2]));
  gtk_plot_data_set_points(dataset[2], px2, py2, dx2, NULL, 8);
 
@@ -342,21 +350,23 @@ build_example2(GtkWidget *active_plot)
 
  gtk_plot_data_set_line_attributes(dataset[2],
                                    GTK_PLOT_LINE_NONE,
-                                   1, &color);
+                                   0, 0, 1, &color);
  gtk_plot_data_set_legend(dataset[2], "V Bars");
 
+ gtk_plot_set_break(GTK_PLOT(plot), GTK_PLOT_AXIS_Y, 0.7, 0.72, .05, 4, GTK_PLOT_SCALE_LINEAR, .6);
 }
 
 int main(int argc, char *argv[]){
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *scrollw1;
- GtkWidget *active_plot;
  GtkWidget *canvas;
- GtkPlotCanvasChild *text;
+ GtkPlotCanvasChild *child;
  GdkColor color;
  gint page_width, page_height;
  gfloat scale = 1.;
+ gchar *custom_labels[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
+ GtkPlotArray *array;
  
  page_width = GTK_PLOT_LETTER_W * scale;
  page_height = GTK_PLOT_LETTER_H * scale;
@@ -382,7 +392,7 @@ int main(int argc, char *argv[]){
  gtk_box_pack_start(GTK_BOX(vbox1),scrollw1, TRUE, TRUE,0);
  gtk_widget_show(scrollw1);
 
- canvas = gtk_plot_canvas_new(page_width, page_height, 1.);
+ canvas = gtk_plot_canvas_new(page_width, page_height, 1.0);
  GTK_PLOT_CANVAS_SET_FLAGS(GTK_PLOT_CANVAS(canvas), GTK_PLOT_CANVAS_DND_FLAGS);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw1), canvas);
 
@@ -401,17 +411,21 @@ int main(int argc, char *argv[]){
  gtk_plot_set_range(GTK_PLOT(active_plot), -1., 1., -1., 1.4);
  gtk_plot_legends_move(GTK_PLOT(active_plot), .500, .05);
  gtk_plot_set_legends_border(GTK_PLOT(active_plot), 0, 0);
- gtk_plot_axis_hide_title(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP);
- gtk_plot_axis_show_ticks(GTK_PLOT(active_plot), 3, 15, 3);
- gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_X, 1., 1);
- gtk_plot_axis_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_Y, 1., 1);
- gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, TRUE);
- gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, TRUE);
+ gtk_plot_axis_hide_title(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP));
+ gtk_plot_axis_show_ticks(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM), 15, 3);
+ gtk_plot_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_X, 1., 1);
+ gtk_plot_set_ticks(GTK_PLOT(active_plot), GTK_PLOT_AXIS_Y, 1., 1);
+ gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP), TRUE);
+ gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT), TRUE);
  gtk_plot_x0_set_visible(GTK_PLOT(active_plot), TRUE);
  gtk_plot_y0_set_visible(GTK_PLOT(active_plot), TRUE);
- gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(active_plot), .15, .06);
- gtk_plot_axis_set_labels_suffix(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT, "%");
+ gtk_plot_axis_set_labels_suffix(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT), "%");
+
+ child = gtk_plot_canvas_plot_new(GTK_PLOT(active_plot));
+ gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .15, .06, .65, .31);
  gtk_widget_show(active_plot);
+ GTK_PLOT_CANVAS_PLOT(child)->flags |= GTK_PLOT_CANVAS_PLOT_SELECT_POINT;
+ GTK_PLOT_CANVAS_PLOT(child)->flags |= GTK_PLOT_CANVAS_PLOT_DND_POINT;
 
  build_example1(active_plot);
 
@@ -431,54 +445,64 @@ int main(int argc, char *argv[]){
  gtk_plot_set_range(GTK_PLOT(active_plot), 0.1 , 100., 0., .85);
  gtk_plot_set_xscale(GTK_PLOT(active_plot), GTK_PLOT_SCALE_LOG10);
 */
- gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP, TRUE);
- gtk_plot_axis_set_visible(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT, TRUE);
+ gtk_plot_axis_set_labels_numbers(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_LEFT), GTK_PLOT_LABEL_FLOAT, 2);
+ gtk_plot_axis_set_labels_numbers(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT), GTK_PLOT_LABEL_FLOAT, 2);
+ gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP), TRUE);
+ gtk_plot_axis_set_visible(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT), TRUE);
  gtk_plot_grids_set_visible(GTK_PLOT(active_plot), TRUE, TRUE, TRUE, TRUE);
- gtk_plot_canvas_add_plot(GTK_PLOT_CANVAS(canvas), GTK_PLOT(active_plot), .15, .4);
- gtk_plot_axis_hide_title(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP);
- gtk_plot_axis_hide_title(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT);
+ gtk_plot_axis_hide_title(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_TOP));
+ gtk_plot_axis_hide_title(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_RIGHT));
  gtk_plot_set_legends_border(GTK_PLOT(active_plot), 2, 3);
  gtk_plot_legends_move(GTK_PLOT(active_plot), .58, .05);
  gtk_widget_show(active_plot);
 
+ child = gtk_plot_canvas_plot_new(GTK_PLOT(active_plot));
+ gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .15, .4, .65, .65);
+ gtk_widget_show(active_plot);
+
  build_example2(active_plot);
 
  gtk_signal_connect(GTK_OBJECT(canvas), "select_item",
                     (GtkSignalFunc) select_item, NULL);
 
+/*
  gtk_signal_connect(GTK_OBJECT(canvas), "move_item",
                     (GtkSignalFunc) move_item, NULL);
+*/
 
-
- gtk_plot_canvas_put_text(GTK_PLOT_CANVAS(canvas), .40, .020, 
-                          "Times-BoldItalic", 16, 0, NULL, NULL, TRUE,
+ child = gtk_plot_canvas_text_new("Times-BoldItalic", 16, 0, NULL, NULL, TRUE,
                           GTK_JUSTIFY_CENTER,
                           "DnD titles, legends and plots");
- gtk_plot_canvas_put_text(GTK_PLOT_CANVAS(canvas), .40, .72,  
-                          "Times-Roman", 16, 0, NULL, NULL, TRUE,
+ gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .40, .020, .0, .0);
+ child = gtk_plot_canvas_text_new("Times-Roman", 16, 0, NULL, NULL, TRUE,
                           GTK_JUSTIFY_CENTER,
                           "You can use \\ssubscripts\\b\\b\\b\\b\\b\\b\\b\\b\\b\\b\\N\\Ssuperscripts");
+ gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .40, .720, .0, .0);
 
- text = gtk_plot_canvas_put_text(GTK_PLOT_CANVAS(canvas), .40, .755,  
-                          "Times-Roman", 16, 0, NULL, NULL, TRUE,
+ child = gtk_plot_canvas_text_new("Times-Roman", 12, 0, NULL, NULL, TRUE,
                           GTK_JUSTIFY_CENTER, 
                           "Format text mixing \\Bbold \\N\\i, italics, \\ggreek \\4\\N and \\+different fonts");
+ gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(canvas), child, .40, .765, .0, .0); 
 
- gtk_plot_text_set_border((GtkPlotText*)text->data, 
-                           GTK_PLOT_BORDER_SHADOW, 2, 0, 2);
+ gtk_plot_text_set_border(&GTK_PLOT_CANVAS_TEXT(child)->text, 
+                          GTK_PLOT_BORDER_SHADOW, 2, 0, 2);
 
  gtk_plot_text_set_border(&GTK_PLOT(active_plot)->left->title, 
                           GTK_PLOT_BORDER_SHADOW, 2, 0, 2);
 
- gtk_widget_show(window1);
- put_child(GTK_PLOT_CANVAS(canvas), .5, .5);
+ array = GTK_PLOT_ARRAY(gtk_plot_array_new(NULL, custom_labels, 12, GTK_TYPE_STRING, FALSE)); 
+ gtk_plot_axis_set_tick_labels(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM), array); 
+ gtk_plot_axis_use_custom_tick_labels(gtk_plot_get_axis(GTK_PLOT(active_plot), GTK_PLOT_AXIS_BOTTOM), TRUE);
 
-/*
- gtk_plot_export_ps (GTK_PLOT(active_plot), "demo.ps", 0, 0, GTK_PLOT_LETTER);
+// put_child(GTK_PLOT_CANVAS(canvas), .5, .5);
+ gtk_widget_show(window1);
+
+ gtk_plot_canvas_export_ps(GTK_PLOT_CANVAS(canvas), "demoplot.ps", GTK_PLOT_PORTRAIT, FALSE, GTK_PLOT_LETTER);
+
+/* 
+ gtk_plot_canvas_export_ps_with_size(GTK_PLOT_CANVAS(canvas), "demoplot.ps", GTK_PLOT_PORTRAIT, TRUE, GTK_PLOT_PSPOINTS, 300, 400);
 */
- gtk_plot_canvas_export_ps(GTK_PLOT_CANVAS(canvas), "demoplot.ps", 0, 0, 
-                           GTK_PLOT_LETTER);
- 
+
  gtk_main();
 
  return(0);
@@ -488,6 +512,7 @@ static void
 put_child(GtkPlotCanvas *canvas, gdouble x, gdouble y)
 {
   GdkColormap *colormap;
+  GtkPlotCanvasChild *child;
   GdkBitmap *mask;
 
   colormap = gdk_colormap_get_system();
@@ -495,6 +520,8 @@ put_child(GtkPlotCanvas *canvas, gdouble x, gdouble y)
   pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
                                                  plot_icon2);
 
-  gtk_plot_canvas_put_pixmap(canvas, pixmap, x, y); 
+  child = gtk_plot_canvas_pixmap_new(pixmap, mask);
+  gtk_plot_canvas_put_child(canvas, child, x, y, x+.05, y+.05); 
+  gdk_pixmap_unref(pixmap);
   gdk_bitmap_unref(mask);
 }
