@@ -293,7 +293,7 @@ clipboard_handler(GtkWidget *widget, GdkEventKey *key)
   if(key->state & GDK_CONTROL_MASK || key->keyval==GDK_Control_L ||
      key->keyval==GDK_Control_R){
     if((key->keyval=='c' || key->keyval == 'C') && sheet->state != GTK_STATE_NORMAL){
-            if(GTK_SHEET_IN_CLIP(sheet)) gtk_sheet_unclip_range(sheet);
+            if(gtk_sheet_in_clip(sheet)) gtk_sheet_unclip_range(sheet);
             gtk_sheet_clip_range(sheet, &sheet->range);
 /*            gtk_sheet_unselect_range(sheet);
 */
@@ -337,16 +337,16 @@ gint change_entry(GtkWidget *widget,
 
   sheet = GTK_SHEET(widget);
 
-  if(*new_col == 0 && col != 0)
+  if(*new_col == 0 && (col != 0 || sheet->state != GTK_STATE_NORMAL))
          gtk_sheet_change_entry(sheet, gtk_combo_get_type());
 
-  if(*new_col == 1 && col != 1)
+  if(*new_col == 1 && (col != 1 || sheet->state != GTK_STATE_NORMAL))
          gtk_sheet_change_entry(sheet, GTK_TYPE_ENTRY);
 
-  if(*new_col == 2 && col != 2)
+  if(*new_col == 2 && (col != 2 || sheet->state != GTK_STATE_NORMAL))
          gtk_sheet_change_entry(sheet, GTK_TYPE_SPIN_BUTTON);
 
-  if(*new_col >= 3 && col < 3)
+  if(*new_col >= 3 && (col < 3 || sheet->state != GTK_STATE_NORMAL))
          gtk_sheet_change_entry(sheet, GTK_TYPE_ITEM_ENTRY);
 
   return TRUE;
@@ -398,7 +398,7 @@ gint alarm_traverse(GtkWidget *widget,
 void show_child(GtkWidget *widget, gpointer data)
 {
  if(!GTK_WIDGET_MAPPED(curve))
-         gtk_sheet_attach(GTK_SHEET(sheets[0]), curve, 2, 7, 0., 0.);
+         gtk_sheet_attach_floating(GTK_SHEET(sheets[0]), curve, 2, 7);
 /*         gtk_sheet_put(GTK_SHEET(sheets[0]), curve, 550, 120);
 */
 }
@@ -437,6 +437,13 @@ build_example1(GtkWidget *widget)
  }
 
  gtk_sheet_row_button_add_label(sheet, 0, "This is\na multiline\nlabel");
+ gtk_sheet_row_button_add_label(sheet, 1, "This is long label");
+
+/*
+ gtk_sheet_column_button_add_label(sheet, 7, "This is\na multiline\nlabel");
+ gtk_sheet_column_button_add_label(sheet, 8, "This is long label");
+*/
+
  gtk_sheet_row_button_justify(sheet, 0, GTK_JUSTIFY_RIGHT);
 
  range.row0=1;
@@ -552,7 +559,7 @@ build_example1(GtkWidget *widget)
 /*      gtk_sheet_put(GTK_SHEET(sheets[0]), bullet[i], 
                     area.x+area.width/2-8, area.y+area.height/2-8);
 */
-      gtk_sheet_attach(GTK_SHEET(sheets[0]), bullet[i], 4+i, 0, .5, .5);
+      gtk_sheet_attach(GTK_SHEET(sheets[0]), bullet[i], 4+i, 0, GTK_EXPAND, GTK_EXPAND, 0, 0);
  }
 
  bullet[5] = gtk_pixmap_new(pixmap, mask);
@@ -561,13 +568,17 @@ build_example1(GtkWidget *widget)
 /* gtk_sheet_put(GTK_SHEET(sheets[0]), bullet[i], 
                area.x+area.width/2-8, area.y+area.height/2-8);
 */
- gtk_sheet_attach(GTK_SHEET(sheets[0]), bullet[5], 10, 0, .5, .5);
+ gtk_sheet_attach(GTK_SHEET(sheets[0]), bullet[5], 10, 0, GTK_EXPAND, GTK_EXPAND, 0, 0);
+ gdk_pixmap_unref(pixmap);
+ gdk_bitmap_unref(mask);
 
  pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
  				   smile_xpm);
  smile = gtk_pixmap_new(pixmap, mask);
  gtk_widget_show(smile);
- gtk_sheet_button_attach(GTK_SHEET(sheets[0]), smile, -1, 5, .5, .5);
+ gtk_sheet_button_attach(GTK_SHEET(sheets[0]), smile, -1, 5);
+ gdk_pixmap_unref(pixmap);
+ gdk_bitmap_unref(mask);
 
 
  show_button=gtk_button_new_with_label("Show me a plot");
@@ -576,7 +587,7 @@ build_example1(GtkWidget *widget)
  gtk_sheet_get_cell_area(GTK_SHEET(sheets[0]), 12, 2, &area);
 /* gtk_sheet_put(GTK_SHEET(sheets[0]), show_button, area.x, area.y);
 */
- gtk_sheet_attach(GTK_SHEET(sheets[0]), show_button, 12, 2, .5, .5);
+ gtk_sheet_attach(GTK_SHEET(sheets[0]), show_button, 12, 2, GTK_FILL, GTK_FILL, 5, 5);
 
  gtk_signal_connect(GTK_OBJECT(show_button), "clicked",
                     (GtkSignalFunc) show_child, 
@@ -597,13 +608,16 @@ build_example2(GtkWidget *widget)
  GtkSheet *sheet;
  GtkSheetRange range;
  GdkColor color;
+ GtkWidget *b;
  
  sheet=GTK_SHEET(widget);
- GTK_SHEET_UNSET_FLAGS(sheet, GTK_SHEET_AUTO_SCROLL);
+ gtk_sheet_set_autoscroll(sheet, FALSE);
 /*
- GTK_SHEET_UNSET_FLAGS(sheet, GTK_SHEET_JUSTIFY_ENTRY);
+ gtk_sheet_set_justify_entry(sheet, FALSE);
 */
+/*
  gtk_sheet_set_selection_mode(sheet, GTK_SELECTION_SINGLE);
+*/
 
  range.row0=0;
  range.rowi=2;
@@ -625,9 +639,12 @@ build_example2(GtkWidget *widget)
  gdk_color_alloc(gtk_widget_get_colormap(widget), &color);
  gtk_sheet_range_set_foreground(sheet, &range, &color);
 
+/*
  gtk_sheet_row_set_sensitivity(sheet, 0, FALSE);
  gtk_sheet_row_set_sensitivity(sheet, 1, FALSE);
  gtk_sheet_row_set_sensitivity(sheet, 2, FALSE);
+*/
+
  gtk_sheet_set_cell(sheet, 0,2, GTK_JUSTIFY_CENTER,
                     "Click the right mouse button to display a popup");
  gtk_sheet_set_cell(sheet, 1,2, GTK_JUSTIFY_CENTER,
@@ -648,6 +665,16 @@ build_example2(GtkWidget *widget)
 		    (GtkSignalFunc) parse_numbers,
                     NULL);                   
 
+ gtk_sheet_set_row_height(GTK_SHEET(sheets[1]), 12, 60);
+ b = gtk_button_new_with_label("GTK_FILL");
+ gtk_sheet_attach(GTK_SHEET(sheets[1]), b, 12, 2, GTK_FILL, GTK_FILL, 5, 5);
+ gtk_widget_show(b);
+ b = gtk_button_new_with_label("GTK_EXPAND");
+ gtk_sheet_attach(GTK_SHEET(sheets[1]), b, 12, 3, GTK_EXPAND, GTK_EXPAND, 5, 5);
+ gtk_widget_show(b);
+ b = gtk_button_new_with_label("GTK_SHRINK");
+ gtk_sheet_attach(GTK_SHEET(sheets[1]), b, 12, 4, GTK_SHRINK, GTK_SHRINK, 5, 5);
+ gtk_widget_show(b);
 }
 
 void
@@ -701,7 +728,7 @@ build_example3(GtkWidget *widget)
  gtk_sheet_range_set_border_color(sheet, &range, &color);
  gtk_sheet_range_set_border(sheet, &range, GTK_SHEET_BOTTOM_BORDER, 4, 1);
 
- GTK_SHEET_SET_FLAGS(sheet, GTK_SHEET_AUTORESIZE);
+ gtk_sheet_set_autoresize(sheet, TRUE);
 
  gtk_sheet_change_entry(sheet, gtk_combo_get_type());
 
@@ -709,6 +736,8 @@ build_example3(GtkWidget *widget)
                     "traverse",
                     (GtkSignalFunc) change_entry, 
                     NULL);
+
+
 }
 
 /*
@@ -1037,49 +1066,43 @@ change_border (GtkWidget *widget, gint border)
 
 
 void 
-change_fg(GtkWidget *widget, gint i, gchar *color_name)
+change_fg(GtkWidget *widget, gint i, GdkColor *color)
 {
    GtkSheet *current;
-   GdkColor color;
    gint cur_page;
    GdkGC *tmp_gc;
 
    cur_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
    current=GTK_SHEET(sheets[cur_page]);
 
-   gdk_color_parse(color_name, &color);
-   gdk_color_alloc(gtk_widget_get_colormap(widget), &color);
-   gtk_sheet_range_set_foreground(current, &current->range, &color);
+   gtk_sheet_range_set_foreground(current, &current->range, color);
 
    tmp_gc=gdk_gc_new(window->window);
-   gdk_gc_set_foreground(tmp_gc,&color);
+   gdk_gc_set_foreground(tmp_gc,color);
    gdk_draw_rectangle(GTK_PIXMAP(fg_pixmap)->pixmap,
-		      tmp_gc, TRUE, 5,20,16,4);
-   gtk_widget_queue_draw(fg_pixmap); gdk_gc_unref(tmp_gc);
+                      tmp_gc, TRUE, 5,20,16,4);
+   gtk_widget_queue_draw(fg_pixmap); 
+   gdk_gc_unref(tmp_gc);
 }
 
 void 
-change_bg(GtkWidget *widget, gint i, gchar *color_name)
+change_bg(GtkWidget *widget, gint i, GdkColor *color)
 {
    GtkSheet *current;
-   GdkColor color;
    gint cur_page;
    GdkGC *tmp_gc;
 
    cur_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
    current=GTK_SHEET(sheets[cur_page]);
 
-   gdk_color_parse(color_name, &color);
-   gdk_color_alloc(gtk_widget_get_colormap(widget), &color);
-   gtk_sheet_range_set_background(current, &current->range, &color);
+   gtk_sheet_range_set_background(current, &current->range, color);
 
    tmp_gc=gdk_gc_new(window->window);
-   gdk_gc_set_foreground(tmp_gc,&color);
+   gdk_gc_set_foreground(tmp_gc,color);
    gdk_draw_rectangle(GTK_PIXMAP(bg_pixmap)->pixmap,
-		      tmp_gc, TRUE, 4,20,18,4);
-   gtk_widget_draw(bg_pixmap, NULL); 
+                      tmp_gc, TRUE, 4,20,18,4);
+   gtk_widget_draw(bg_pixmap, NULL);
    gdk_gc_unref(tmp_gc);
-
 }
 
 void 
@@ -1226,7 +1249,7 @@ int main(int argc, char *argv[])
   gtk_widget_set_usize(GTK_FONT_COMBO(font_combo)->italic_button, 32, 32);
   gtk_widget_set_usize(GTK_FONT_COMBO(font_combo)->bold_button, 32, 32);
   gtk_widget_show(font_combo);
-  gtk_signal_connect(GTK_OBJECT(font_combo), "changed", new_font, NULL);
+  gtk_signal_connect(GTK_OBJECT(font_combo), "changed", GTK_SIGNAL_FUNC(new_font), NULL);
 
   gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
@@ -1302,8 +1325,9 @@ int main(int argc, char *argv[])
   gtk_box_pack_start(GTK_BOX(status_box), location, FALSE, TRUE, 0);
   gtk_widget_show(location);
 
-  entry=gtk_entry_new(); gtk_box_pack_start(GTK_BOX(status_box), entry,
-  TRUE, TRUE, 0); gtk_widget_show(entry);
+  entry=gtk_entry_new(); 
+  gtk_box_pack_start(GTK_BOX(status_box), entry, TRUE, TRUE, 0); 
+  gtk_widget_show(entry);
 
   notebook=gtk_notebook_new();
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
@@ -1357,18 +1381,24 @@ int main(int argc, char *argv[])
   tpixmap = gtk_pixmap_new(pixmap, mask);
   gtk_container_add(GTK_CONTAINER(left_button), tpixmap);
   gtk_widget_show(tpixmap);
+  gdk_pixmap_unref(pixmap);
+  gdk_bitmap_unref(mask);
 
   pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
 				    center_just);
   tpixmap = gtk_pixmap_new(pixmap, mask);
   gtk_container_add(GTK_CONTAINER(center_button), tpixmap);
   gtk_widget_show(tpixmap);
+  gdk_pixmap_unref(pixmap);
+  gdk_bitmap_unref(mask);
 
   pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
 				    right_just);
   tpixmap = gtk_pixmap_new(pixmap, mask);
   gtk_container_add(GTK_CONTAINER(right_button), tpixmap);
   gtk_widget_show(tpixmap);
+  gdk_pixmap_unref(pixmap);
+  gdk_bitmap_unref(mask);
 
   pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
 				    paint);
@@ -1378,6 +1408,8 @@ int main(int argc, char *argv[])
 		    bg_pixmap);
 
   gtk_widget_show(bg_pixmap);
+  gdk_pixmap_unref(pixmap);
+  gdk_bitmap_unref(mask);
 
   pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
 				    font);
@@ -1388,6 +1420,8 @@ int main(int argc, char *argv[])
 		    fg_pixmap);
 
   gtk_widget_show(fg_pixmap);
+  gdk_pixmap_unref(pixmap);
+  gdk_bitmap_unref(mask);
 
   gtk_widget_show(window);
 
